#ifndef _CI_CONFIG_H_
#define _CI_CONFIG_H_

#include <cstring>

#include <iostream.h>
#include "global.h"

#define	MAX_NUM_ZENITH_DIVISION		10
#define	MAX_NUM_AZMUTHAL_DIVISION	16

#define MULTI_HIST_ZENITH_DIVISION	8
#define MULTI_HIST_AZMUTHAL_DIVISION	8

//image geometry setup
class CiConfigImage {
public:
	//the clipping radius.
	int	radius;
	//the indexes of the center point of the image, count from lower-left
	//corner and start from (0,0).
	int	center_x;
	int	center_y;
	//the maximum measurable width and height of the image
	int max_msrbl_image_x;
	int max_msrbl_image_y;
	char type[4];

	CiConfigImage()
	{
		//assign default values. values will be changed based on config file.
		max_msrbl_image_x = min(DEFAULT_MAX_IMAGE_X, DEFAULT_MAX_IMAGE_Y);
		max_msrbl_image_y = max_msrbl_image_x;
		radius = max_msrbl_image_x / 2;
		center_x = DEFAULT_MAX_IMAGE_X / 2;
		center_y = DEFAULT_MAX_IMAGE_Y / 2;
		strcpy(type,"jpg");
	}
};

class CiConfigGeometry {
public:
	//geometry setup:
	int num_of_zenith_division;
	int num_of_azmuthal_division;
	int view_range;
	int starting_zenith_angle;
	int ending_zenith_angle;
	int threshhold_level;
	double PARzerocal;
	double PARspancal;
	double Fleckszerocal;
	double Flecksspancal;
	double Latitude;
	int Month1;
	int Day1;
	int Month2;
	int Day2;

	BOOL blankout_flag;
	int	blankout1;
	int	blankout2;

	CiConfigGeometry()
	{
		num_of_zenith_division = 5;
		num_of_azmuthal_division = 4;
		view_range = 150;
		starting_zenith_angle = 0;
		ending_zenith_angle = 75;
		threshhold_level = 50;
		PARzerocal = -540.0;
		PARspancal = 10.4;
		Fleckszerocal = -720.0;
		Flecksspancal = 0.340;
		Latitude = 30;
		Month1=3;
		Day1=21;
		Month2=9;
		Day2=21;

		blankout_flag = FALSE;
		blankout1 = 0;
		blankout2 = 0;
	}

	void Print()
	{
		cout << "num_of_zenith_division: " << num_of_zenith_division << endl;
		cout << "num_of_azmuthal_division: " << num_of_azmuthal_division << endl;
		cout << "view_range: " << view_range << endl;
		cout << "starting_zenith_angle: " << starting_zenith_angle << endl;
		cout << "ending_zenith_angle: " << ending_zenith_angle << endl;
		cout << "threshhold_level: " << threshhold_level << endl;
		cout << "PARzerocal: " << PARzerocal << endl;
		cout << "PARspancal: " << PARspancal << endl;
		cout << "Fleckszerocal: " << Fleckszerocal << endl;
		cout << "Flecksspancal: " << Flecksspancal << endl;
		cout << "Latitude: " << Latitude << endl;
		cout << "First Suntrack Month: " << Month1;
		cout << "  Day: " << Day1 << endl;
		cout << "Last Suntrack Month: " << Month2;
		cout << "  Day: " << Day2 << endl;
		cout << "blankout_flag: " << blankout_flag << endl;
		cout << "blankout1: " << blankout1 << endl;
		cout << "blankout2: " << blankout2 << endl;
	}
};

class CiConfigOutput {
public:
	//output setup:
	BOOL bLAI;			//Leaf Area Index
	BOOL bLD;			//Leaf Distribution
	BOOL bMFIA;			//Mean Foliage Inclination Angle
	BOOL bTR;			//Transmission Coef. for Radiation Penetration
	BOOL bTD;			//Transmission Coef. for Diffuse penetration
	BOOL bK;			//Extinction Coef.
	BOOL bPAR;			//PAR reading
	BOOL bSUNTRACK;		//Want suntrack?

	CiConfigOutput()
	{
		bLAI = TRUE;
		bLD = TRUE;
		bMFIA = TRUE;
		bTR = TRUE;
		bTD = TRUE;
		bK = TRUE;
		bPAR = TRUE;
		bSUNTRACK = TRUE;
	}

	void Print()
	{
		cout << "bLAI: " << bLAI << endl;
		cout << "bLD: " << bLD << endl;
		cout << "bMFIA: " << bMFIA << endl;
		cout << "bTR: " << bTR << endl;
		cout << "bTD: " << bTD << endl;
		cout << "bK: " << bK << endl;
		cout << "bPAR: " << bPAR << endl;
		cout << "bSUNTRACK: " << bSUNTRACK << endl;
	}

} ;

class CiConfigNetwork {
public:
	CString serialPort;
	CString baudrate;
	CString imageFileDir;

	CiConfigNetwork()
	{
		serialPort = "COM1";
		baudrate = "9600";
		imageFileDir = "C:";
	}

	void Print()
	{
		cout << "serialPort: " << serialPort << endl;
		cout << "baudrate: " << baudrate << endl;
		cout << "imageFileDir: " << imageFileDir << endl;
	}
} ;

class CiConfigMeasure {
public:
	int		cutoffLow;
	int		cutoffHigh;
	int		cutoff;
	BOOL	discardLow;
	BOOL	discardHigh;

	CiConfigMeasure()
	{
		cutoffLow = 0;
		cutoffHigh = 100;
		cutoff = 50;
		discardLow = FALSE;
		discardHigh = FALSE;
	}

	void Print()
	{
		cout << "cutoffLow: " << cutoffLow << endl;
		cout << "cutoffHigh: " << cutoffHigh << endl;
		cout << "cutoff: " << cutoff << endl;
		cout << "discardLow: " << discardLow << endl;
		cout << "discardHigh: " << discardHigh << endl;
	}
};

#endif