// CIDPreMsrDialog.cpp : implementation file
//

#include "stdafx.h"
#include "dib.h"
#include "CIDPreMsrDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CiConfigGeometry* ci_config_geometry;
extern CMainFrame* mainFrame;

/////////////////////////////////////////////////////////////////////////////
// CCIDPreMsrDialog dialog


CCIDPreMsrDialog::CCIDPreMsrDialog(CDibDoc* pDoc, CWnd* pParent /*=NULL*/)
	: CDialog(CCIDPreMsrDialog::IDD, pParent)
{

	//{{AFX_DATA_INIT(CCIDPreMsrDialog)
	m_blankout_end = 0;
	m_blankout_start = 0;
	m_threshhold_level = 0;
	//}}AFX_DATA_INIT

	this->pDoc = pDoc;
	m_blankout_flag = FALSE;
	blankout_applied = FALSE;
	m_Initialized = FALSE;
}


void CCIDPreMsrDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCIDPreMsrDialog)
	DDX_Control(pDX, IDC_PRE_MSR_BUTTON_OK, m_OKButton);
	DDX_Control(pDX, IDC_PRE_MSR_BUTTON_CANCEL, m_CancelButton);
	DDX_Control(pDX, IDC_PRE_MSR_BUTTON_APPLY, m_ApplyButton);
	DDX_Control(pDX, IDC_PRE_MSR_BLKT_START, m_BlankoutStartEditCtrl);
	DDX_Control(pDX, IDC_PRE_MSR_BLKT_END, m_BlankoutEndEditCtrl);
	DDX_Control(pDX, IDC_PRE_MSR_SPIN_BLKT_END, m_BlankoutEndCtrl);
	DDX_Control(pDX, IDC_PRE_MSR_SPIN_BLKT_START, m_BlankoutStartCtrl);
	DDX_Text(pDX, IDC_PRE_MSR_BLKT_END, m_blankout_end);
	DDV_MinMaxInt(pDX, m_blankout_end, 0, 359);
	DDX_Text(pDX, IDC_PRE_MSR_BLKT_START, m_blankout_start);
	DDV_MinMaxInt(pDX, m_blankout_start, 0, 359);
	DDX_Text(pDX, IDC_PRE_MSR_THRESH_LEVEL, m_threshhold_level);
	DDV_MinMaxInt(pDX, m_threshhold_level, 0, 100);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CCIDPreMsrDialog, CDialog)
	//{{AFX_MSG_MAP(CCIDPreMsrDialog)
	ON_EN_CHANGE(IDC_PRE_MSR_BLKT_END, OnChangePreMsrBlktEnd)
	ON_EN_CHANGE(IDC_PRE_MSR_BLKT_START, OnChangePreMsrBlktStart)
	ON_EN_CHANGE(IDC_PRE_MSR_THRESH_LEVEL, OnChangePreMsrThreshLevel)
	ON_BN_CLICKED(IDC_PRE_MSR_BUTTON_OK, OnPreMsrButtonOk)
	ON_BN_CLICKED(IDC_PRE_MSR_BUTTON_CANCEL, OnPreMsrButtonCancel)
	ON_BN_CLICKED(IDC_PRE_MSR_BUTTON_APPLY, OnPreMsrButtonApply)
	ON_BN_CLICKED(IDC_PRE_MSR_RADIO_NO, OnPreMsrRadioNo)
	ON_BN_CLICKED(IDC_PRE_MSR_RADIO_YES, OnPreMsrRadioYes)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCIDPreMsrDialog message handlers

BOOL CCIDPreMsrDialog::DestroyWindow() 
{
	pDoc->m_drawBlankoutAsLines = FALSE;
	POSITION pos = pDoc->GetFirstViewPosition();
	CDibView* pFirstView = (CDibView*)pDoc->GetNextView( pos );
	pFirstView->DrawBlankoutRegion(m_blankout_start, m_blankout_end);
	
	return CDialog::DestroyWindow();
}

void CCIDPreMsrDialog::OnChangePreMsrBlktEnd() 
{
	if (!m_Initialized)
		return;

	UpdateData(TRUE);

	mainFrame->setConfigChanged(TRUE);

	blankout_applied = FALSE;
	
	DrawBlankoutRegion();

	m_ApplyButton.EnableWindow(TRUE);
}

void CCIDPreMsrDialog::OnChangePreMsrBlktStart() 
{
	if (!m_Initialized)
		return;

	UpdateData(TRUE);

	mainFrame->setConfigChanged(TRUE);

	blankout_applied = FALSE;
	
	DrawBlankoutRegion();

	m_ApplyButton.EnableWindow(TRUE);
}

void CCIDPreMsrDialog::OnChangePreMsrThreshLevel() 
{
	if (!m_Initialized)
		return;

	m_ApplyButton.EnableWindow(TRUE);
}

void CCIDPreMsrDialog::OnPreMsrButtonOk() 
{
	pDoc->m_drawBlankoutAsLines = FALSE;
	POSITION pos = pDoc->GetFirstViewPosition();
	CDibView* pFirstView = (CDibView*)pDoc->GetNextView( pos );
	pFirstView->DrawBlankoutRegion(m_blankout_start, m_blankout_end);

	EndDialog(IDOK);
	OnPreMsrButtonApply();
}

void CCIDPreMsrDialog::OnPreMsrButtonCancel() 
{
	pDoc->m_drawBlankoutAsLines = FALSE;
	POSITION pos = pDoc->GetFirstViewPosition();
	CDibView* pFirstView = (CDibView*)pDoc->GetNextView( pos );
	pFirstView->DrawBlankoutRegion(m_blankout_start, m_blankout_end);

	EndDialog(IDCANCEL);
}

void CCIDPreMsrDialog::OnPreMsrButtonApply() 
{
	//update the member variables from their controls.
	if (!UpdateData(TRUE))
		return;

	//if blankout flag has been changed from original value, need apply.
	if (m_blankout_flag != ci_config_geometry->blankout_flag)
	{
		blankout_applied = FALSE;
		ci_config_geometry->blankout_flag = m_blankout_flag;
	}
	
	if (m_blankout_flag)
	{
		if (m_blankout_start != ci_config_geometry->blankout1)
		{
			blankout_applied = FALSE;
			ci_config_geometry->blankout1 = m_blankout_start;
		}
		if (m_blankout_end != ci_config_geometry->blankout2)
		{
			blankout_applied = FALSE;
			ci_config_geometry->blankout2 = m_blankout_end;
		}
	}

	if (ci_config_geometry->threshhold_level != m_threshhold_level)
	{
		ci_config_geometry->threshhold_level = m_threshhold_level;
		blankout_applied = FALSE;
	}

	//if (!blankout_applied)
	{
		::SetCursor(::LoadCursor(NULL, IDC_WAIT));
		pDoc->m_drawBlankoutAsLines = FALSE;
		pDoc->SetState(CI_SHOWLEAFIMAGE);
		pDoc->UpdateImages();
		pDoc->UpdateAllViews(NULL);
		::SetCursor(::LoadCursor(NULL, IDC_ARROW));
	}

	blankout_applied = TRUE;
	m_ApplyButton.EnableWindow(FALSE);
}

/*
* Draw the blankout region on the view. The view will cover the blankout region
* with lines.
*/
void CCIDPreMsrDialog::DrawBlankoutRegion()
{
	ci_config_geometry->blankout_flag = m_blankout_flag;

	if (ci_config_geometry->blankout_flag)
	{
		ci_config_geometry->blankout1 = m_blankout_start;
		ci_config_geometry->blankout2 = m_blankout_end;
	}

	// if it is drawing the image as a processed image, need to change it to unprocessed.
	if (pDoc->GetState() == CI_SHOWLEAFIMAGE)
	{
		pDoc->SetState(CI_SHOWCROPPEDIMAGE);
		pDoc->UpdateImages();
	}
	POSITION pos = pDoc->GetFirstViewPosition();
	CDibView* pFirstView = (CDibView*)pDoc->GetNextView( pos );
	pDoc->m_drawBlankoutAsLines = TRUE;
	pFirstView->DrawBlankoutRegion(m_blankout_start, m_blankout_end);
}

void CCIDPreMsrDialog::UpdateState()
{
	m_blankout_flag = ci_config_geometry->blankout_flag;

	if (ci_config_geometry->blankout_flag)
	{
		m_blankout_start = ci_config_geometry->blankout1;
		m_blankout_end = ci_config_geometry->blankout2;
	}
	else
	{
		m_blankout_start = ci_config_geometry->blankout1;
		m_blankout_end = ci_config_geometry->blankout2;
	}

	m_threshhold_level = ci_config_geometry->threshhold_level;
}

/*
* This is called when the "No" radio button is clicked.
*/
void CCIDPreMsrDialog::OnPreMsrRadioNo() 
{
	m_blankout_flag = FALSE;
	EnableControlsForBlankout(m_blankout_flag);
	m_ApplyButton.EnableWindow(TRUE);
	DrawBlankoutRegion();
}

/*
* This is called when the "Yes" radio button is clicked.
*/
void CCIDPreMsrDialog::OnPreMsrRadioYes() 
{
	m_blankout_flag = TRUE;
	EnableControlsForBlankout(m_blankout_flag);
	m_ApplyButton.EnableWindow(TRUE);
	DrawBlankoutRegion();
}

/*
* Enable/Disable controls according to blankout flag.
*/
void CCIDPreMsrDialog::EnableControlsForBlankout(BOOL blankoutFlag)
{
	m_BlankoutStartEditCtrl.EnableWindow(blankoutFlag);
	m_BlankoutEndEditCtrl.EnableWindow(blankoutFlag);
	m_BlankoutStartCtrl.EnableWindow(blankoutFlag);
	m_BlankoutEndCtrl.EnableWindow(blankoutFlag);
}

BOOL CCIDPreMsrDialog::OnInitDialog() 
{
	m_Initialized = FALSE;

	CDialog::OnInitDialog();

	m_BlankoutStartCtrl.SetRange(0, 359);
	m_BlankoutStartCtrl.SetBase(10);
	m_BlankoutStartCtrl.SetBuddy(&m_BlankoutStartEditCtrl);
	m_BlankoutStartCtrl.SetPos(m_blankout_start);
	int pos1 = m_BlankoutStartCtrl.GetPos();
	m_BlankoutEndCtrl.SetRange(0, 359);
	m_BlankoutEndCtrl.SetBase(10);
	m_BlankoutEndCtrl.SetBuddy(&m_BlankoutEndEditCtrl);
	m_BlankoutEndCtrl.SetPos(m_blankout_end);
	int pos2 = m_BlankoutEndCtrl.GetPos();

	if (m_blankout_flag)
	{
		((CButton*)GetDlgItem(IDC_PRE_MSR_RADIO_YES))->SetCheck(1);
		((CButton*)GetDlgItem(IDC_PRE_MSR_RADIO_NO))->SetCheck(0);
		DrawBlankoutRegion();
	}
	else
	{
		((CButton*)GetDlgItem(IDC_PRE_MSR_RADIO_YES))->SetCheck(0);
		((CButton*)GetDlgItem(IDC_PRE_MSR_RADIO_NO))->SetCheck(1);
	}
	EnableControlsForBlankout(m_blankout_flag);
	/*
	*/
	m_threshhold_level = ci_config_geometry->threshhold_level;

	m_Initialized = TRUE;

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}
