// DibDoc.cpp : implementation of the CDibDoc class
//

#include "stdafx.h"
#include "Dib.h"
//#include "DibBlankoutDialog.h"
#include "CIDPreMsrDialog.h"

#include "DibDoc.h"
#include "DibView.h"
#include "Mainfrm.h"
#include "IResultDialog.h"
#include "CiConfig.h"
#include <vfw.h>

extern "C" {
#include "jpeglib.h"
}
#include <setjmp.h>
#include <fcntl.h>      /* Needed only for _O_RDWR definition */
#include <math.h>
#include <io.h>
#include <stdlib.h>
#include <stdio.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define CI_PI 3.14159265

extern CMainFrame* mainFrame;

extern CiConfigGeometry* ci_config_geometry;
extern CiConfigImage* ci_config_image;
extern CiConfigOutput* ci_config_output;
extern CiConfigMeasure* ci_config_measure;


extern double ci_cos(double x);
extern double ci_sin(double x);
extern double ci_sqrt(double x);
extern double ci_pow(double z, double x);
extern double ci_log(double x);
extern double ci_exp(double x);
extern double ci_tan(double x);
extern double ci_atan(double x);

extern double PAR,Flecks;

struct my_error_mgr {
  struct jpeg_error_mgr pub;	/* "public" fields */

  jmp_buf setjmp_buffer;	/* for return to caller */
};

typedef struct my_error_mgr * my_error_ptr;


/*
 * Here's the routine that will replace the standard error_exit method:
 */

METHODDEF(void)
my_error_exit (j_common_ptr cinfo)
{
  /* cinfo->err really points to a my_error_mgr struct, so coerce pointer */
  my_error_ptr myerr = (my_error_ptr) cinfo->err;

  /* Always display the message. */
  /* We could postpone this until after returning, if we chose. */
  (*cinfo->err->output_message) (cinfo);

  /* Return control to the setjmp point */
  longjmp(myerr->setjmp_buffer, 1);
}


/////////////////////////////////////////////////////////////////////////////
// CDibDoc

IMPLEMENT_DYNCREATE(CDibDoc, CDocument)

BEGIN_MESSAGE_MAP(CDibDoc, CDocument)
	//{{AFX_MSG_MAP(CDibDoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDibDoc construction/destruction

CDibDoc::CDibDoc()
: original_hdib(NULL), cropped_hdib(NULL), leaf_hdib(NULL)
{
	m_drawBlankoutAsLines = FALSE;
	viewImageScale = (float)1.0;// (5./6.);
	m_fillWhenPrint = TRUE;
}

CDibDoc::~CDibDoc()
{
	SetState(CI_INITIALIZED);
}

BOOL CDibDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	SetState(CI_INITIALIZED);

	return TRUE;
}

void CDibDoc::SetOriginalDib(HGLOBAL hdib)
{
	if (hdib != original_hdib)
	{
		SetState(CI_INITIALIZED); //This will clean all the old dibs.

		original_hdib = hdib;

		if (original_hdib)
		{
			BOOL config_changed = FALSE;

			// Do consistance checking. The crop circle should be within the
			// image boundary.
			LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)GlobalLock(original_hdib);
			if (lpbi != NULL)
			{
				if (lpbi->biWidth < ci_config_image->radius+ci_config_image->center_x
					|| ci_config_image->center_x < ci_config_image->radius
					|| lpbi->biHeight < ci_config_image->radius+ci_config_image->center_y
					|| ci_config_image->center_y < ci_config_image->radius)
				{
					ci_config_image->center_x = lpbi->biWidth/2;
					ci_config_image->center_y = lpbi->biHeight/2;
					if (ci_config_image->center_x > ci_config_image->center_y)
						ci_config_image->radius = ci_config_image->center_y;
					else
						ci_config_image->radius = ci_config_image->center_x;
					mainFrame->setConfigChanged(TRUE);
					config_changed = TRUE;
				}
			}
			GlobalUnlock(original_hdib);

			// update the images.
			//
			cropped_hdib = copyBitMap(original_hdib);
			OnCrop(cropped_hdib);
			SetState(CI_SHOWCROPPEDIMAGE);

			if (config_changed)
			{
				LPCTSTR lpszText = "Setup has been adjusted to fit with new image. Image center and radius have been changed.";
				AfxMessageBox(lpszText, MB_OK);
			}
		}

		UpdateAllViews(NULL);
	}
}


unsigned CDibDoc::RealizePalette(CDC* pDC)
{
	unsigned uRemaps = 0;
	HDRAWDIB hdd = DrawDibOpen();
	if (hdd) {
		uRemaps = DrawDibRealize(hdd, *pDC, FALSE); 
		DrawDibClose(hdd);
	}
	return uRemaps;
}

//rc is the client area that bitmap will be drawn.
//if bitmap is larger than the rc, than bitmap is trimed.
//if bitmap is smaller than the rc, bitmap is centered in rc.
//the image center (specified by ci_config_image->center_x and .._y)
//should be put at the center of rc;
//note: the image center is in the image cordinate.
void CDibDoc::OnDraw(CDC* pDC, CRect rc)
{
	LPBITMAPINFOHEADER lpbi = NULL;
	switch (state) {
	case CI_INITIALIZED: break;
	case CI_SHOWRAWIMAGE: 
		lpbi = (LPBITMAPINFOHEADER)GlobalLock(original_hdib);
		break;
	case CI_SHOWCROPPEDIMAGE: 
		lpbi = (LPBITMAPINFOHEADER)GlobalLock(cropped_hdib);
		break;
	case CI_SHOWLEAFIMAGE: 
		lpbi = (LPBITMAPINFOHEADER)GlobalLock(leaf_hdib);
		break;
	}

	if (lpbi)
	{
		BYTE* lpBits = (BYTE*)lpbi + lpbi->biSize;

		/*
		* The following code is to calculate the mapping between image source area and
		* destination area. The destination area is given by CRect rc. The source area
		* is selected so that the center of the image cource area is mapped to the 
		* center of the destination area. Then, according to the source/destination
		* ratio, the source area boundary is calculated.
		* All the calculation is based on device coordinates. That is the upper left
		* corner of image is (0,0);
		*/
		int src_x, src_y, src_dx, src_dy;
		int dst_x = rc.left, dst_y = rc.top, dst_dx = rc.Width(), dst_dy = rc.Height();
		float src_center_x, src_center_y, dst_center_x, dst_center_y;
		float ratio_x = (float)(1./viewImageScale), ratio_y = (float)(1./viewImageScale); //ratio of sizes (source/destination)
		int x, y;

		src_center_x = (float) ci_config_image->center_x;
		//The biHeight is in pixels and coordinate starts from 0.
		src_center_y = (float) (lpbi->biHeight - ci_config_image->center_y - 1);
		//This should never happen, but...
		if (src_center_y < 0) src_center_y = 0.;
		else if (src_center_y >= lpbi->biHeight) src_center_y = (float)(lpbi->biHeight - 1);

		dst_center_x = (float)(dst_x + dst_dx / 2.);
		dst_center_y = (float)(dst_y + dst_dy / 2.);

		//Calculate which region in image needs to be copied to which client area.
		src_x = (int) (src_center_x - ratio_x * (dst_center_x - dst_x));
		if (src_x < 0)
		{
			//The left boundary of image is right of the left boundary of destination.
			src_x = 0;
			//adjust the destination left boundary.
			x = (int) (dst_center_x - (src_center_x - src_x) / ratio_x);
			dst_dx = dst_dx - (x - dst_x);
			dst_x = x;
		}
		src_y = (int) (src_center_y - ratio_y * (dst_center_y - dst_y));
		if (src_y < 0)
		{
			//The upper boundary of image is below the upper boundary of destination.
			src_y = 0;
			//adjust the destination upper boundary.
			y = (int) (dst_center_y - (src_center_y - src_y) / ratio_y);
			dst_dy = dst_dy - (y - dst_y);
			dst_y = y;
		}

		int tmp_x, tmp_y; //lower right corner of the source area.
		tmp_x = (int) (src_center_x + ratio_x * (dst_x + dst_dx - dst_center_x));
		if (tmp_x > lpbi->biWidth)
		{
			//The right boundary of image is left of the right boundary of destination.
			tmp_x = lpbi->biWidth;
			//adjust the destination left boundary.
			x = (int) ((tmp_x - src_center_x) / ratio_x); //distance between right and center.
			dst_dx = (int) (dst_center_x + x - dst_x);
		}
		src_dx = tmp_x - src_x;
		tmp_y = (int) (src_center_y + ratio_y * (dst_y + dst_dy - dst_center_y));
		if (tmp_y > lpbi->biHeight)
		{
			//The bottom boundary of image is above the bottom boundary of destination.
			tmp_y = lpbi->biHeight;
			//adjust the destination bottom boundary.
			y = (int) ((tmp_y - src_center_y) / ratio_y); //distance between bottom and center.
			dst_dy = (int) (dst_center_y + y - dst_y);
		}
		src_dy = tmp_y - src_y;


		HDRAWDIB hdd = DrawDibOpen();
		if (hdd) {
			DrawDibDraw(hdd,
				pDC->m_hDC,
				dst_x, dst_y,						// upper left corner of destination
				dst_dx, dst_dy,						// width, height
				lpbi,	
				lpBits,	
				src_x, src_y,						// upper left corner of source in DC.
				src_dx, src_dy,	
				0);
			DrawDibClose(hdd);
		} else {
			TRACE0("DrawDibOpen failed\n");
		}
	}
	switch (state) {
	case CI_INITIALIZED: break;
	case CI_SHOWRAWIMAGE: 
		GlobalUnlock(original_hdib);
		break;
	case CI_SHOWCROPPEDIMAGE: 
		GlobalUnlock(cropped_hdib);
		break;
	case CI_SHOWLEAFIMAGE: 
		GlobalUnlock(leaf_hdib);
		break;
	}

	DrawGeometryDivision(pDC, rc);
}

/////////////////////////////////////////////////////////////////////////////
// CDibDoc serialization

void CDibDoc::Serialize(CArchive& ar)
{
	CString fn = ar.GetFile()->GetFilePath();
	
	LPCTSTR filename = (LPCTSTR) fn;
	BITMAPFILEHEADER		bfh;
	LPBITMAPINFOHEADER		lpbi;
	CiImageFormat			imageFormat = CI_UNKNOWN_FORMAT;

	if (ar.IsStoring())
	{

		if (original_hdib)
		{
			if (saveProcessedImageWhenSave)
				lpbi = (LPBITMAPINFOHEADER)GlobalLock(leaf_hdib);
			else
				lpbi = (LPBITMAPINFOHEADER)GlobalLock(original_hdib);
			imageFormat = CI_JPEG_FORMAT;
			int nlen = strlen(filename);
			if (nlen > 3)
			{
			    if (_strnicmp(filename+nlen-3, "bmp", 3)==0)
				{
					imageFormat = CI_BITMAP_FORMAT;
					if(strcmp(ci_config_image->type,"bmp")!=0)
					{
						strcpy(ci_config_image->type,"bmp");
						mainFrame->setConfigChanged(TRUE);
					}

				}
				else if(strcmp(ci_config_image->type,"jpg")!=0)
				{
					strcpy(ci_config_image->type,"jpg");
					mainFrame->setConfigChanged(TRUE);
				}

			}
			if (imageFormat == CI_BITMAP_FORMAT)
			{
				int		nBPP = lpbi->biBitCount;
				int		nColors = (int)lpbi->biClrUsed;

				// figure out actual size of color table
				if (nColors == 0 && nBPP <= 8) {
					nColors = (1 << nBPP);
				}
				if (lpbi->biCompression == BI_RGB) {
					// uncompressed bitmap, image size might not be set
					DWORD dwBytesPerRow = (((lpbi->biWidth * nBPP) + 31) / 32) * 4;
					lpbi->biSizeImage = dwBytesPerRow * lpbi->biHeight;
					}

				// Set up BMP header.
				bfh.bfType = 0x4d42;                // "BM"
				bfh.bfReserved1 = 0;
				bfh.bfReserved2 = 0;
				bfh.bfOffBits = sizeof(BITMAPFILEHEADER) +
								sizeof(BITMAPINFOHEADER) +
								sizeof(RGBQUAD) * nColors;
				bfh.bfSize = bfh.bfOffBits + lpbi->biSizeImage;
				ar.Write(&bfh, sizeof bfh);
				ar.Write(lpbi, bfh.bfSize - sizeof bfh);
				if (saveProcessedImageWhenSave)
					GlobalUnlock(leaf_hdib);
				else
					GlobalUnlock(original_hdib);
			}
			else if (imageFormat == CI_JPEG_FORMAT)
			{
				/* This struct contains the JPEG compression parameters and pointers to
				* working space (which is allocated as needed by the JPEG library).
				* It is possible to have several such structures, representing multiple
				* compression/decompression processes, in existence at once.  We refer
				* to any one struct (and its associated working data) as a "JPEG object".
				*/
				struct jpeg_compress_struct cinfo;
				/* This struct represents a JPEG error handler.  It is declared separately
				* because applications often want to supply a specialized error handler
				* (see the second half of this file for an example).  But here we just
				* take the easy way out and use the standard error handler, which will
				* print a message on stderr and call exit() if compression fails.
				* Note that this struct must live as long as the main JPEG parameter
				* struct, to avoid dangling-pointer problems.
				*/
				struct jpeg_error_mgr jerr;
				/* More stuff */
				FILE * outfile;		/* target file */
				JSAMPROW row_pointer[1];	/* pointer to JSAMPLE row[s] */
				int row_stride;		/* physical row width in image buffer */
				int image_width = lpbi->biWidth;
				int image_height = lpbi->biHeight;

				/* Step 1: allocate and initialize JPEG compression object */

				/* We have to set up the error handler first, in case the initialization
				* step fails.  (Unlikely, but it could happen if you are out of memory.)
				* This routine fills in the contents of struct jerr, and returns jerr's
				* address which we place into the link field in cinfo.
				*/
				cinfo.err = jpeg_std_error(&jerr);
				/* Now we can initialize the JPEG compression object. */
				jpeg_create_compress(&cinfo);

				/* Step 2: specify data destination (eg, a file) */
				/* Note: steps 2 and 3 can be done in either order. */

				char temp_filename[MAX_PATH];
				char temp_pathname[MAX_PATH];
				if (GetTempPath(MAX_PATH, temp_pathname) == 0)
					AfxThrowArchiveException(CArchiveException::generic);
				if (GetTempFileName(&temp_pathname[0], "CID", 0, &temp_filename[0]) == 0)
					AfxThrowArchiveException(CArchiveException::generic);
				/* Here we use the library-supplied code to send compressed data to a
				* stdio stream.  You can also write your own code to do something else.
				* VERY IMPORTANT: use "b" option to fopen() if you are on a machine that
				* requires it in order to write binary files.
				*/
				if ((outfile = fopen(temp_filename, "wb")) == NULL) {
					fprintf(stderr, "can't open %s\n", filename);
					AfxThrowArchiveException(CArchiveException::generic);
				}
				jpeg_stdio_dest(&cinfo, outfile);

				/* Step 3: set parameters for compression */

				/* First we supply a description of the input image.
				* Four fields of the cinfo struct must be filled in:
				*/
				cinfo.image_width = image_width; 	/* image width and height, in pixels */
				cinfo.image_height = image_height;
				cinfo.input_components = 3;		/* # of color components per pixel */
				cinfo.in_color_space = JCS_RGB; 	/* colorspace of input image */
				/* Now use the library's routine to set default compression parameters.
				* (You must set at least cinfo.in_color_space before calling this,
				* since the defaults depend on the source color space.)
				*/
				jpeg_set_defaults(&cinfo);
				/* Now you can set any non-default parameters you wish to.
				* Here we just illustrate the use of quality (quantization table) scaling:
				*/
				jpeg_set_quality(&cinfo, 75, TRUE /* limit to baseline-JPEG values */);

				/* Step 4: Start compressor */

				/* TRUE ensures that we will write a complete interchange-JPEG file.
				* Pass TRUE unless you are very sure of what you're doing.
				*/
				jpeg_start_compress(&cinfo, TRUE);

				/* Step 5: while (scan lines remain to be written) */
				/*           jpeg_write_scanlines(...); */

				/* Here we use the library's state variable cinfo.next_scanline as the
				* loop counter, so that we don't have to keep track ourselves.
				* To keep things simple, we pass one scanline per call; you can pass
				* more if you wish, though.
				*/
				row_stride = image_width * 3;	/* JSAMPLEs per row in image_buffer */
				DWORD dwBytesPerRow = (((cinfo.image_width * 24) + 31) / 32) * 4;
				row_pointer[0] = (BYTE*)lpbi + sizeof(BITMAPINFOHEADER) + lpbi->biSizeImage;
				BYTE* rowhead = (BYTE*) malloc(dwBytesPerRow);
				while (cinfo.next_scanline < cinfo.image_height) 
				{
					/* jpeg_write_scanlines expects an array of pointers to scanlines.
					* Here the array is only one element long, but you could pass
					* more than one scanline at a time if that's more convenient.
					*/
					row_pointer[0] -= dwBytesPerRow;
					memcpy(rowhead, row_pointer[0], dwBytesPerRow);
					BYTE tmp;
					BYTE* tmp_p;
					for (int i=0; i<cinfo.image_width; i++)
					{
						tmp_p = rowhead + i * 3;
						tmp = *tmp_p;
						*tmp_p = *(tmp_p + 2);
						*(tmp_p + 2) = tmp;
					}
					(void) jpeg_write_scanlines(&cinfo, &rowhead, 1);
				}
				free(rowhead);

				/* Step 6: Finish compression */

				jpeg_finish_compress(&cinfo);
				/* After finish_compress, we can close the output file. */
				fflush(outfile);
				fclose(outfile);
				/* Step 7: release JPEG compression object */

				/* This is an important step since it will release a good deal of memory. */
				jpeg_destroy_compress(&cinfo);

				/* And we're done! */

				if ((outfile = fopen(temp_filename, "rb")) == NULL)
				{
					AfxThrowArchiveException(CArchiveException::generic);
				}

				char buffer[1];
				int numread;
				/* Read in input: */
				while( numread = fread( buffer, sizeof( char ), 1, outfile ) > 0 )
					ar.Write(buffer, numread);
				fclose(outfile);
				try {
					CFile::Remove(temp_filename);
				} catch (CFileException e) {}
			}
		}

	}
	else
	{

		if (ar.Read(&bfh, sizeof bfh) != sizeof bfh) 
		{
			AfxThrowArchiveException(CArchiveException::endOfFile);
		}
		if (bfh.bfType != 0x4d42 ||
			bfh.bfReserved1 != 0 || bfh.bfReserved2 != 0 ||
			bfh.bfOffBits < (sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER)))
		{
			imageFormat = CI_JPEG_FORMAT;
			//AfxThrowArchiveException(CArchiveException::badIndex);
			if(strcmp(ci_config_image->type,"jpg")!=0)
			{
				strcpy(ci_config_image->type,"jpg");
				mainFrame->setConfigChanged(TRUE);
			}

		}
		else
		{
			imageFormat = CI_BITMAP_FORMAT;
			if(strcmp(ci_config_image->type,"bmp")!=0)
			{
				strcpy(ci_config_image->type,"bmp");
				mainFrame->setConfigChanged(TRUE);
			}

		}
		
		//The handle to bitmap read in from file or converted from other format.
		HGLOBAL hdib = 0;
		if (imageFormat == CI_BITMAP_FORMAT)
		{
			unsigned uDibSize = bfh.bfSize - sizeof bfh;
			hdib = ::GlobalAlloc(GMEM_MOVEABLE, uDibSize);
			if (!hdib) {
				AfxThrowArchiveException(CArchiveException::generic);
			}
			lpbi = (LPBITMAPINFOHEADER)::GlobalLock(hdib);
			if (!lpbi) {
				AfxThrowArchiveException(CArchiveException::generic);
			}
			if (ar.Read(lpbi, uDibSize) != uDibSize) {
				::GlobalUnlock(hdib);
				::GlobalFree(hdib);
				AfxThrowArchiveException(CArchiveException::endOfFile);
			}
			::GlobalUnlock(hdib);
		}
		else if (imageFormat == CI_JPEG_FORMAT)
		{
			/* This struct contains the JPEG decompression parameters and pointers to
			* working space (which is allocated as needed by the JPEG library).
			*/
			struct jpeg_decompress_struct cinfo;
			/* We use our private extension JPEG error handler.
			* Note that this struct must live as long as the main JPEG parameter
			* struct, to avoid dangling-pointer problems.
			*/
			struct my_error_mgr jerr;
			/* More stuff */
			FILE * infile;		/* source file */
			JSAMPARRAY buffer;		/* Output row buffer */
			int row_stride;		/* physical row width in output buffer */

			/* In this example we want to open the input file before doing anything else,
			* so that the setjmp() error recovery below can assume the file is open.
			* VERY IMPORTANT: use "b" option to fopen() if you are on a machine that
			* requires it in order to read binary files.
			*/

			if ((infile = fopen(filename, "rb")) == NULL) 
			{
				AfxThrowArchiveException(CArchiveException::generic);
			}

			/* Step 1: allocate and initialize JPEG decompression object */

			/* We set up the normal JPEG error routines, then override error_exit. */
			cinfo.err = jpeg_std_error(&jerr.pub);
			jerr.pub.error_exit = my_error_exit;
			/* Establish the setjmp return context for my_error_exit to use. */
			if (setjmp(jerr.setjmp_buffer))
			{
				/* If we get here, the JPEG code has signaled an error.
				* We need to clean up the JPEG object, close the input file, and return.
				*/
				jpeg_destroy_decompress(&cinfo);
				fclose(infile);

				AfxThrowArchiveException(CArchiveException::generic);
			}
			/* Now we can initialize the JPEG decompression object. */
			jpeg_create_decompress(&cinfo);

			/* Step 2: specify data source (eg, a file) */

			jpeg_stdio_src(&cinfo, infile);

			/* Step 3: read file parameters with jpeg_read_header() */

			(void) jpeg_read_header(&cinfo, TRUE);
			/* We can ignore the return value from jpeg_read_header since
			*   (a) suspension is not possible with the stdio data source, and
			*   (b) we passed TRUE to reject a tables-only JPEG file as an error.
			* See libjpeg.doc for more info.
			*/

			/* Step 4: set parameters for decompression */

			/* In this example, we don't need to change any of the defaults set by
			* jpeg_read_header(), so we do nothing here.
			*/

			/* Step 5: Start decompressor */

			(void) jpeg_start_decompress(&cinfo);
			/* We can ignore the return value since suspension is not possible
			* with the stdio data source.
			*/
			if (cinfo.output_components != 3 && cinfo.output_components != 1)
				AfxThrowArchiveException(CArchiveException::badIndex);

			DWORD dwBytesPerRow = (((cinfo.image_width * 24) + 31) / 32) * 4;
			unsigned uDibSize = sizeof(BITMAPINFOHEADER) + dwBytesPerRow * cinfo.image_height;
			hdib = ::GlobalAlloc(GMEM_MOVEABLE, uDibSize);
			LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)::GlobalLock(hdib);
			lpbi->biSize = 40;
			lpbi->biWidth = cinfo.image_width;
			lpbi->biHeight = cinfo.image_height;
			lpbi->biPlanes = 1;
			lpbi->biBitCount = 24;
			lpbi->biCompression = BI_RGB;
			lpbi->biSizeImage = dwBytesPerRow * lpbi->biHeight;
			lpbi->biXPelsPerMeter = 0;
			lpbi->biYPelsPerMeter = 0;
			lpbi->biClrUsed = 0;
			lpbi->biClrImportant = 0;

			/* We may need to do some setup of our own at this point before reading
			* the data.  After jpeg_start_decompress() we have the correct scaled
			* output image dimensions available, as well as the output colormap
			* if we asked for color quantization.
			* In this example, we need to make an output work buffer of the right size.
			*/ 
			/* JSAMPLEs per row in output buffer */
			row_stride = cinfo.output_width * cinfo.output_components;
			/* Make a one-row-high sample array that will go away when done with image */
			buffer = (*cinfo.mem->alloc_sarray)
			((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);

			/* Step 6: while (scan lines remain to be read) */
			/*           jpeg_read_scanlines(...); */

			/* Here we use the library's state variable cinfo.output_scanline as the
			* loop counter, so that we don't have to keep track ourselves.
			*/
			BYTE* rowhead = (BYTE*)lpbi + sizeof(BITMAPINFOHEADER) + lpbi->biSizeImage;
			while (cinfo.output_scanline < cinfo.output_height)
			{
				/* jpeg_read_scanlines expects an array of pointers to scanlines.
				* Here the array is only one element long, but you could ask for
				* more than one scanline at a time if that's more convenient.
				*/
				(void) jpeg_read_scanlines(&cinfo, buffer, 1);
				/* Assume put_scanline_someplace wants a pointer and sample count. */
				if (cinfo.output_components == 3)
				{
					rowhead -= dwBytesPerRow;
					memcpy(rowhead, buffer[0], row_stride);
					//switch r and b color.
					BYTE tmp;
					BYTE* tmp_p;
					for (int i=0; i<cinfo.output_width; i++)
					{
						tmp_p = rowhead + i * 3;
						tmp = *tmp_p;
						*tmp_p = *(tmp_p + 2);
						*(tmp_p + 2) = tmp;
					}
				}
				else if(cinfo.output_components == 1)
				{
					rowhead -= dwBytesPerRow;
					BYTE tmp;
					BYTE* tmp_p;
					for (int i=0; i<cinfo.output_width; i++)
						{
						tmp_p = rowhead + i * 3;
						tmp = *(buffer[0] + i);
						*tmp_p = tmp;
						*(tmp_p + 1) = tmp;
						*(tmp_p + 2) = tmp;
					}
				}
			}
			GlobalUnlock(hdib);

			/* Step 7: Finish decompression */

			(void) jpeg_finish_decompress(&cinfo);
			/* We can ignore the return value since suspension is not possible
			* with the stdio data source.
			*/

			/* Step 8: Release JPEG decompression object */

			/* This is an important step since it will release a good deal of memory. */
			jpeg_destroy_decompress(&cinfo);

			/* After finish_decompress, we can close the input file.
			* Here we postpone it until after no more JPEG errors are possible,
			* so as to simplify the setjmp error logic above.  (Actually, I don't
			* think that jpeg_destroy can do an error exit, but why assume anything...)
			*/
			fclose(infile);

			/* At this point you may want to check to see whether any corrupt-data
			* warnings occurred (test whether jerr.pub.num_warnings is nonzero).
			*/

			/* And we're done! */
		}
		SetState(CI_INITIALIZED);

		SetOriginalDib(hdib);
	}
}

/////////////////////////////////////////////////////////////////////////////
// CDibDoc diagnostics

#ifdef _DEBUG
void CDibDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CDibDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDibDoc commands

void CDibDoc::SetState(CiState s)
{
	if (s == CI_INITIALIZED)
	{
		m_drawGeometryDivision = TRUE;

		if (original_hdib) {
			GlobalFree(original_hdib);
			original_hdib = NULL;
		}
		if (cropped_hdib) {
			GlobalFree(cropped_hdib);
			cropped_hdib = NULL;
		}
		if (leaf_hdib) {
			GlobalFree(leaf_hdib);
			leaf_hdib = NULL;
		}

		//initialize the starting and ending indices to be
		//invalid values.
		int i;
		for (i=0; i<MAX_IMAGE_HEIGHT; i++)
			CropCircleStartIndex[i] = -1;
		for (i=0; i<MAX_IMAGE_HEIGHT; i++)
			CropCircleEndIndex[i] = -1;
		for (i=0; i<MAX_IMAGE_HEIGHT; i++)
			StartIndexScan1[i] = -1;
		for (i=0; i<MAX_IMAGE_HEIGHT; i++)
			EndIndexScan1[i] = -1;
		for (i=0; i<MAX_IMAGE_HEIGHT; i++)
			StartIndexScan2[i] = -1;
		for (i=0; i<MAX_IMAGE_HEIGHT; i++)
			EndIndexScan2[i] = -1;
		for (i=0; i<MAX_IMAGE_HEIGHT; i++)
			BlankoutStartI[i] = -1;
		for (i=0; i<MAX_IMAGE_HEIGHT; i++)
      {
         BlankoutEndI[i] = -1;
         BlankoutEndI2[i] = -1;
			BlankoutStartI2[i] = -1;
      }

		memset(croppedDibRows, 0, MAX_IMAGE_HEIGHT * sizeof(COLORPOINT*));
		memset(leafDibRows, 0, MAX_IMAGE_HEIGHT * sizeof(COLORPOINT*));
	}

	state = s;
}

void CDibDoc::UpdateImages()
{
	if (state == CI_INITIALIZED || state == CI_SHOWRAWIMAGE)
		return;

	if (cropped_hdib && cropped_hdib!=original_hdib)
	{
		GlobalFree(cropped_hdib);
		cropped_hdib = NULL;
	}
	if (leaf_hdib)
	{
		GlobalFree(leaf_hdib);
		leaf_hdib = NULL;
	}

	cropped_hdib = copyBitMap(original_hdib);
	OnCrop(cropped_hdib);

	if (state == CI_SHOWLEAFIMAGE)
	{
		if (ci_config_geometry->blankout_flag)
			BlankoutImage();
		GetLeafDib();
	}
}


void CDibDoc::DrawGeometryDivision(CDC* pDC, CRect rc)
{
//	char line[20];

	int rc_center_x = rc.left + (int)(rc.Width()/2.+0.5);
	int rc_center_y = rc.top + (int)(rc.Height()/2.+0.5);
	float ratio = (float)(1./viewImageScale);
	//if (m_drawGeometryDivision)
	int radius_min = (int)(ci_config_image->radius/ratio * ci_config_geometry->starting_zenith_angle * 2./ci_config_geometry->view_range);
	int radius_max = (int)(ci_config_image->radius/ratio * ci_config_geometry->ending_zenith_angle * 2./ci_config_geometry->view_range);
	if (TRUE)
	{
		pDC->SelectStockObject(HOLLOW_BRUSH);
	
		double dr = ((double)(radius_max - radius_min)) / ci_config_geometry->num_of_zenith_division;
		for (int i=ci_config_geometry->num_of_zenith_division; i>=0; i--)
		{
			int radius = (int) (radius_min + dr * i);
			CRect	circle(rc_center_x-radius, rc_center_y-radius,
						   rc_center_x+radius, rc_center_y+radius);
			pDC->Ellipse(circle);
		}
	//	char line[20];
		for (int j=0; j<ci_config_geometry->num_of_azmuthal_division; j++)
		{
			double angle = (double)2.*CI_PI*j/ci_config_geometry->num_of_azmuthal_division+CI_PI/2.0;//start azimuthal divisions at N 5/8/06
		//	if(!ci_config_output->bSUNTRACK)
			{
				int dx = (int) (radius_min * ci_cos(angle));
				int dy = (int) (radius_min * ci_sin(angle));
				pDC->MoveTo(rc_center_x + dx, rc_center_y - dy);
				dx = (LONG) (radius_max * ci_cos(angle));
				dy = (LONG) (radius_max * ci_sin(angle));
				//device cordinates start at upper left, use - for dy.
				CPoint dest(rc_center_x + dx, rc_center_y - dy);
				pDC->LineTo(dest);
			}
	//		else
	//		{
	//			int dx = (int) (radius_min * (-ci_sin(angle)));
	//			int dy = (int) (radius_min * ci_cos(angle));
	//			pDC->MoveTo(rc_center_x + dx, rc_center_y - dy);
	//			dx = (LONG) (radius_max * (-ci_sin(angle)));
	//			dy = (LONG) (radius_max * ci_cos(angle));
	//			//device cordinates start at upper left, use - for dy.
	//			CPoint dest(rc_center_x + dx, rc_center_y - dy);
	//			pDC->LineTo(dest);
	//		}
		}
	}

	if (m_drawBlankoutAsLines && ci_config_geometry->blankout_flag)
	{
		CBrush* pOldBrush;
		CPen* pOldPen;

		LOGPEN m_logpen;
		m_logpen.lopnStyle = PS_SOLID;
		m_logpen.lopnWidth.x = 1;
		m_logpen.lopnWidth.y = 1;
		m_logpen.lopnColor = RGB(255, 255, 128);


		LOGBRUSH m_logbrush;
		m_logbrush.lbStyle = BS_HOLLOW;
		//m_logbrush.lbColor = RGB(0, 0, 0);
		//m_logbrush.lbHatch = HS_HORIZONTAL;

		CBrush brush;
		if (brush.CreateBrushIndirect(&m_logbrush))
			pOldBrush = pDC->SelectObject(&brush);
		else
			pOldBrush = (CBrush*)pDC->SelectStockObject(NULL_BRUSH);
		CPen pen;
		if (pen.CreatePenIndirect(&m_logpen))
			pOldPen = pDC->SelectObject(&pen);
		else
			pOldPen = (CPen*)pDC->SelectStockObject(NULL_PEN);
	
		int radius_min = (int)(ci_config_image->radius/ratio * ci_config_geometry->starting_zenith_angle * 2./ci_config_geometry->view_range);
		int radius_max = (int)(ci_config_image->radius/ratio * ci_config_geometry->ending_zenith_angle * 2./ci_config_geometry->view_range);
		int	m_blankout_start = ci_config_geometry->blankout1;
		int m_blankout_end = ci_config_geometry->blankout2;
		int da = m_blankout_end - m_blankout_start;
		if (da < 0)
			da += 360;
		int endangle = m_blankout_start + da +90; //added 90 for change from W=0 to N=0 before V3.1.0.1 Aug3,2005
		//draw a line every 3 degree.
		for (int j=m_blankout_start+90; j<=endangle; j++) //added 90 for change from W=0 to N=0 before V3.1.0.1 Aug3,2005
		{
			if ((j/3)*3==j || j==m_blankout_start || j==endangle)
			{
				double angle = (double)2.*CI_PI*j/360;
				int dx = (int) (radius_min * ci_cos(angle));
				int dy = (int) (radius_min * ci_sin(angle));
				pDC->MoveTo(rc_center_x + dx, rc_center_y - dy);
				dx = (LONG) (radius_max * ci_cos(angle));
				dy = (LONG) (radius_max * ci_sin(angle));
				//device cordinates start at upper left, use - for dy.
				CPoint dest(rc_center_x + dx, rc_center_y - dy);
				pDC->LineTo(dest);
			}
		}
		pDC->SelectObject(pOldBrush);
		pDC->SelectObject(pOldPen);
	}
    if(ci_config_output->bSUNTRACK)
	{
		pDC->SelectStockObject(HOLLOW_BRUSH);

		char outmsg[255];
		double solar_declination;
		double hour_of_day;
		double solar_elevation=0.0;
		double solar_elevation_deg;
		double solar_azimuth;
		double solarpathStep;
		int iSolarpathStep2=0;
		double solar_declination1;
		double solar_declination2;
		int dayOfYear;
		bool MoveToFlag;
		double view_angle=ci_config_geometry->ending_zenith_angle-ci_config_geometry->starting_zenith_angle;
//	char line[255];
		strcpy(outmsg,"test");
		dayOfYear=0;
		if(ci_config_geometry->Month1>1)dayOfYear=31;
		if(ci_config_geometry->Month1>2)dayOfYear+=28;
		if(ci_config_geometry->Month1>3)dayOfYear+=31;
		if(ci_config_geometry->Month1>4)dayOfYear+=30;
		if(ci_config_geometry->Month1>5)dayOfYear+=31;
		if(ci_config_geometry->Month1>6)dayOfYear+=30;
		if(ci_config_geometry->Month1>7)dayOfYear+=31;
		if(ci_config_geometry->Month1>8)dayOfYear+=31;
		if(ci_config_geometry->Month1>9)dayOfYear+=30;
		if(ci_config_geometry->Month1>10)dayOfYear+=31;
		if(ci_config_geometry->Month1>11)dayOfYear+=30;
		dayOfYear+=ci_config_geometry->Day1;
		solar_declination1=23.458/180*CI_PI*cos(1.0*(dayOfYear-172.0)/182.5*CI_PI);
		solarpathStep=23.458/180*CI_PI-solar_declination1;

		for(solar_declination=23.458/180*CI_PI;solar_declination>-23.459/180*CI_PI;solar_declination-=solarpathStep)//Summer in Northern Hemisphere solar_declination-=23.458/180*CI_PI
		{
//		sprintf(outmsg,"Day %d, SolarD %f, SolarD1 %f, step %f",dayOfYear,solar_declination,solar_declination1,solarpathStep);
//		AfxMessageBox(outmsg);
		MoveToFlag=true;
			if(iSolarpathStep2==1)
			{
				dayOfYear=0;
				if(ci_config_geometry->Month2>1)dayOfYear=31;
				if(ci_config_geometry->Month2>2)dayOfYear+=28;
				if(ci_config_geometry->Month2>3)dayOfYear+=31;
				if(ci_config_geometry->Month2>4)dayOfYear+=30;
				if(ci_config_geometry->Month2>5)dayOfYear+=31;
				if(ci_config_geometry->Month2>6)dayOfYear+=30;
				if(ci_config_geometry->Month2>7)dayOfYear+=31;
				if(ci_config_geometry->Month2>8)dayOfYear+=31;
				if(ci_config_geometry->Month2>9)dayOfYear+=30;
				if(ci_config_geometry->Month2>10)dayOfYear+=31;
				if(ci_config_geometry->Month2>11)dayOfYear+=30;
				dayOfYear+=ci_config_geometry->Day2;
				solar_declination2=23.458/180*CI_PI*cos(1.0*(dayOfYear-172.0)/182.5*CI_PI);
				solarpathStep=solar_declination1-solar_declination2;
			}
			else if(iSolarpathStep2==2)solarpathStep=solar_declination2+23.458/180*CI_PI;
			iSolarpathStep2++;
		for(hour_of_day=.1;hour_of_day<23.95;hour_of_day+=0.10)
		{
			int dx;
			int dy;
			solar_elevation=(sin(ci_config_geometry->Latitude/180*CI_PI)*sin(solar_declination)+cos(ci_config_geometry->Latitude/180*CI_PI)*cos(solar_declination)*cos(.261799383*(hour_of_day-12)));
			if(solar_elevation<1.0 && solar_elevation>-1)solar_elevation=asin(solar_elevation);
			else if(solar_elevation>=1.0)solar_elevation=1.570796327;
			      else solar_elevation=3.0*1.570796327;
			solar_azimuth=((sin(solar_declination)-sin(solar_elevation)*sin(ci_config_geometry->Latitude/180*CI_PI))/(cos(solar_elevation)*cos(ci_config_geometry->Latitude/180*CI_PI)));
			if(solar_azimuth<=-1.0)solar_azimuth=CI_PI;
			else if(solar_azimuth>=1.0)solar_azimuth=0;
			     else solar_azimuth=acos(solar_azimuth);
			if(hour_of_day>12)
			{
				solar_azimuth=2*CI_PI-solar_azimuth;
			}
			solar_elevation_deg=solar_elevation*180/CI_PI;
			if(solar_elevation_deg>(90.0-ci_config_geometry->ending_zenith_angle) && MoveToFlag)
			{
				MoveToFlag=false;
//				dx = (int) (radius_max * (-ci_sin(solar_azimuth))); //cos(theta+90)=-sin(theta)
//				dy = (int) (radius_max * ci_cos(solar_azimuth));
				dx = (LONG) ((radius_max-(radius_max-radius_min)/view_angle*(solar_elevation_deg-90+ci_config_geometry->ending_zenith_angle)) * (-sin(solar_azimuth)));
				dy = (LONG) ((radius_max-(radius_max-radius_min)/view_angle*(solar_elevation_deg-90+ci_config_geometry->ending_zenith_angle)) * cos(solar_azimuth));
				CPoint dest(rc_center_x + dx, rc_center_y - dy);
				pDC->MoveTo(dest);
			}
			else
				if(solar_elevation_deg>(90.0-ci_config_geometry->ending_zenith_angle))
				{
					dx = (LONG) ((radius_max-(radius_max-radius_min)/view_angle*(solar_elevation_deg-90+ci_config_geometry->ending_zenith_angle)) * (-sin(solar_azimuth)));
					dy = (LONG) ((radius_max-(radius_max-radius_min)/view_angle*(solar_elevation_deg-90+ci_config_geometry->ending_zenith_angle)) * cos(solar_azimuth));
//	sprintf(line, "dx %d  dy %d el %f  az %f ",(int)dx,(int)dy,solar_elevation,solar_azimuth  );
//	if(dx<1 && dx>-1 && dy<1 && dy>-1)AfxMessageBox((LPCTSTR) line, MB_OK);
					//device cordinates start at upper left, use - for dy.
					CPoint dest(rc_center_x + dx, rc_center_y - dy);
					pDC->LineTo(dest);
					if(hour_of_day/(int)(hour_of_day+.5)<1.002 && hour_of_day/(int)(hour_of_day+.5)>0.998)
					{
						
						pDC->LineTo((int)(rc_center_x + dx),(int)( rc_center_y-dy - 3));
						pDC->LineTo((int)(rc_center_x + dx),(int)( rc_center_y-dy + 3));
						pDC->LineTo((int)(rc_center_x + dx),(int)( rc_center_y-dy));
					}
				}
			}
		}
	}
}

//return TRUE if point(i,j) is in cropping circle.
//i, j are the index in DIB counting from lower-left corner.
BOOL CDibDoc::IsInCropCircle(int i, int j) 
{
	LONG d_square = (LONG) (i - ci_config_image->center_x)*(i - ci_config_image->center_x)
						+ (j - ci_config_image->center_y)*(j - ci_config_image->center_y);
	LONG r_square = ci_config_image->radius * ci_config_image->radius;
	if (d_square <= r_square)
		return TRUE;
	else
		return FALSE;
}

//return the region index for point(i,j). i, j are indexs counting from lower left corner
//of image.
int CDibDoc::RingIndex(int i, int j)
{
	int d = Radius(i, j);
	if (d > ci_config_image->radius)
		return -1;
   // Added the following to avoid blankout problems in calculations 5/10/06 JRG
   				if (ci_config_geometry->blankout_flag
					&& BlankoutStartI[j] >=0
					&& BlankoutStartI[j] <= i
					&& i <= BlankoutEndI[j])
					return -1;
   				if (ci_config_geometry->blankout_flag
					&& BlankoutStartI2[j] >=0
					&& BlankoutStartI2[j] <= i
					&& i <= BlankoutEndI2[j])
					return -1;


	int radius_min = (int)(ci_config_image->radius * ci_config_geometry->starting_zenith_angle * 2./ci_config_geometry->view_range);
	int radius_max = (int)(ci_config_image->radius * ci_config_geometry->ending_zenith_angle * 2./ci_config_geometry->view_range);
	if (d > radius_max || d < radius_min)
		return -1;
	double dr = ((double)(radius_max - radius_min)) / ci_config_geometry->num_of_zenith_division;

	int index = (int) ((d - radius_min) / dr);
	if (index == ci_config_geometry->num_of_zenith_division)
		index --;

	return index;
}

void CDibDoc::OnCrop(HGLOBAL cropping_hdib)
{
	LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)GlobalLock(cropping_hdib);
	int		nBPP = lpbi->biBitCount;
	int		nColors = (int)lpbi->biClrUsed;
	// figure out actual size of color table
	if (nColors == 0 && nBPP <= 8) {
		nColors = (1 << nBPP);
	}
	DWORD dwBytesPerRow = (((lpbi->biWidth * nBPP) + 31) / 32) * 4;
	if (lpbi->biCompression == BI_RGB) {
		// uncompressed bitmap, image size might not be set
		lpbi->biSizeImage = dwBytesPerRow * lpbi->biHeight;
	}

	//now begin cropping the image using the cropping circle.
	unsigned char* ptr = (unsigned char*) lpbi;
	//pointing to the image data.
	ptr = ptr + sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * nColors;
	for (int i=0; i<lpbi->biHeight; i++)
		croppedDibRows[i] = (COLORPOINT *) (ptr + i * dwBytesPerRow);

	//remember in bitmap, the cordinates start at lower left corner, while
	//in device cordinates, it starts at upper left corner.
	if (lpbi->biBitCount == 24) // this should be true.
	{
		for (int j=0; j<lpbi->biHeight; j++)
		{
			CropCircleStartIndex[j] = -1;
			CropCircleEndIndex[j] = -1;

			for (int i=0; i<lpbi->biWidth; i++)
			{
				if (IsInCropCircle(i,j))
				{
					if (CropCircleStartIndex[j] == -1)
						CropCircleStartIndex[j] = i;
					CropCircleEndIndex[j] = i;
				}
				else
				{
					croppedDibRows[j][i].r = 0xFF;
					croppedDibRows[j][i].g = 0xFF;
					croppedDibRows[j][i].b = 0xFF;
				}
			}// end i loop
		}// end j loop
	}
	else
	{
		//
	}

	UpdateScanIndex();

	GlobalUnlock(cropping_hdib);
}

/*
* calculate the scan start/end indices.
* a scan line can be broken into two segments due to the start/end zenith angle.
*/
void CDibDoc::UpdateScanIndex()
{
	//The radius of the two circle specified by starting/ending zenith angle.
	int outer_r = ci_config_image->radius * ci_config_geometry->ending_zenith_angle * 2 
					/ ci_config_geometry->view_range;
	int inner_r = ci_config_image->radius * ci_config_geometry->starting_zenith_angle * 2
					/ ci_config_geometry->view_range;
	//cout << "outer_r = " << outer_r << ", inner_r = " << inner_r << endl;
	int dx_out = -1, dx_in = -1;
	int dy = 0;
   int tmpstore;

	for (int j=0; j<MAX_IMAGE_HEIGHT; j++)
	{
		StartIndexScan1[j] = -1;
		EndIndexScan1[j] = -1;
		StartIndexScan2[j] = -1;
		EndIndexScan2[j] = -1;

		if (CropCircleStartIndex[j] < 0)
			continue;

		dx_out = -1;
		dx_in = -1;

		dy = j-ci_config_image->center_y;
		if ((outer_r*outer_r - dy*dy) >= 0)
		{
			//interset the outer circle.
			dx_out = (int)ci_sqrt(outer_r*outer_r - dy*dy);
			if ((inner_r*inner_r - dy*dy) >= 0)
				dx_in = (int)ci_sqrt(inner_r*inner_r - dy*dy);
		}

		if (dx_out > 0 && dx_in > 0)
		{
			//intersect with both circle.
			StartIndexScan1[j] = ci_config_image->center_x - dx_out;
			EndIndexScan1[j] = ci_config_image->center_x - dx_in;
			StartIndexScan2[j] = ci_config_image->center_x + dx_in;
			EndIndexScan2[j] = ci_config_image->center_x + dx_out;
			//restrain everything within crop circle.
			if (StartIndexScan1[j] < CropCircleStartIndex[j])
				StartIndexScan1[j] = CropCircleStartIndex[j];
			if (EndIndexScan1[j] < CropCircleStartIndex[j])
			{
				StartIndexScan1[j] = StartIndexScan2[j];
				EndIndexScan1[j] = EndIndexScan2[j];
				StartIndexScan2[j] = -1;
				EndIndexScan2[j] = -1;

				if (StartIndexScan1[j] < CropCircleStartIndex[j])
					StartIndexScan1[j] = CropCircleStartIndex[j];
				if (EndIndexScan1[j] < CropCircleStartIndex[j])
				{
					StartIndexScan1[j] = -1;
					EndIndexScan1[j] = -1;
				}
			}

			if (StartIndexScan1[j] != -1 && StartIndexScan1[j] > CropCircleEndIndex[j])
			{
				StartIndexScan1[j] = -1;
				EndIndexScan1[j] = -1;
				StartIndexScan2[j] = -1;
				EndIndexScan2[j] = -1;
			}
			if (EndIndexScan1[j] != -1 && EndIndexScan1[j] > CropCircleEndIndex[j])
				EndIndexScan1[j] = CropCircleEndIndex[j];

			if (StartIndexScan2[j] != -1 && StartIndexScan2[j] > CropCircleEndIndex[j])
			{
				StartIndexScan2[j] = -1;
				EndIndexScan2[j] = -1;
			}
			if (EndIndexScan2[j] != -1 && EndIndexScan2[j] > CropCircleEndIndex[j])
				EndIndexScan2[j] = CropCircleEndIndex[j];
		}
		else if (dx_out > 0 && dx_in <= 0)
		{
			//interset with outer circle only.
			StartIndexScan1[j] = ci_config_image->center_x - dx_out;
			EndIndexScan1[j] = ci_config_image->center_x + dx_out;
			if (StartIndexScan1[j] < CropCircleStartIndex[j])
				StartIndexScan1[j] = CropCircleStartIndex[j];
			if (EndIndexScan1[j] > CropCircleEndIndex[j])
				EndIndexScan1[j] = CropCircleEndIndex[j];
		}

		BlankoutStartI[j] = -1;
		BlankoutEndI[j] = -1;
		BlankoutStartI2[j] = -1; // 5/8/06 allows two blackout regions on one line.
		BlankoutEndI2[j] = -1;

		for (int i=CropCircleStartIndex[j]; i<=CropCircleEndIndex[j]; i++)
		{
			//calculate the blankout region.
			if (ci_config_geometry->blankout_flag)
			{
				if (IsInBlankoutRegion(i, j))
				{
					if (BlankoutStartI[j] == -1)
						BlankoutStartI[j] = i;
					BlankoutEndI[j] = i;
				}
            else
            {
               if(BlankoutEndI[j] == i-1 && i>0)
               {
                  BlankoutStartI2[j] = BlankoutStartI[j];
                  BlankoutEndI2[j]=BlankoutEndI[j];
                  BlankoutStartI[j]=-1;
                  BlankoutEndI[j]=-1;
               }
            }
         }
		}

      tmpstore=BlankoutEndI2[j];
      BlankoutEndI2[j]=BlankoutEndI[j];
      BlankoutEndI[j]=tmpstore;
      tmpstore=BlankoutStartI2[j];
      BlankoutStartI2[j]=BlankoutStartI[j];
      BlankoutStartI[j]=tmpstore;

	}// end j loop
/*
	for (int jj=0; jj<MAX_IMAGE_HEIGHT; jj++)
	{
		if (CropCircleStartIndex[jj] >= 0)
		cout << jj << "\t"
			<< "(" << StartIndexScan1[jj] << "," << EndIndexScan1[jj] << ")\t"
			<<  "(" << StartIndexScan2[jj] << "," << EndIndexScan2[jj] << ")\t"
			<<  "(" << BlankoutStartI[jj] << "," << BlankoutEndI[jj] << ")\n";
	}
*/
}

//blank out the cropped image.
void CDibDoc::BlankoutImage()
{
	int i, j;
	for (j=0; j<MAX_IMAGE_HEIGHT; j++)
	{
		// if it not within the crop circle, no data is there.
		if (CropCircleStartIndex[j] == -1) continue;

		// if it is not out of ending zenith angle, treat it as blanked out.
		if (StartIndexScan1[j] == -1)
		{
			for (i=CropCircleStartIndex[j]; i<=CropCircleEndIndex[j]; i++)
			{
				croppedDibRows[j][i].r >>= 1;
				croppedDibRows[j][i].g >>= 1;
				croppedDibRows[j][i].b >>= 1;
			}
		}
		else
		{
			for (i=CropCircleStartIndex[j]; i<=CropCircleEndIndex[j]; i++)
			{
				BOOL inBlankout = true;

				//if it is on either of the scan lines, it is not blanked out.
				if (StartIndexScan1[j] <= i && i <= EndIndexScan1[j])
					inBlankout = false;
				else if (StartIndexScan2[j] >=0 && StartIndexScan2[j] <= i && i <= EndIndexScan2[j])
					inBlankout = false;

				//but if it is in blankout region, it is always blanked out.
				if (ci_config_geometry->blankout_flag
					&& BlankoutStartI[j] >=0
					&& BlankoutStartI[j] <= i
					&& i <= BlankoutEndI[j])
					inBlankout = true;

            if (ci_config_geometry->blankout_flag
					&& BlankoutStartI2[j] >=0
					&& BlankoutStartI2[j] <= i
					&& i <= BlankoutEndI2[j])
					inBlankout = true;


				if (inBlankout)
				{
					croppedDibRows[j][i].r >>= 1;
					croppedDibRows[j][i].g >>= 1;
					croppedDibRows[j][i].b >>= 1;
				}
			}
		}//end else
	}// end j loop
}

HGLOBAL CDibDoc::copyBitMap(HGLOBAL src_hdib)
{
	LPBITMAPINFOHEADER src_lpbi = (LPBITMAPINFOHEADER)GlobalLock(src_hdib);
	int		nBPP = src_lpbi->biBitCount;
	int		nColors = (int)src_lpbi->biClrUsed;
	// figure out actual size of color table
	if (nColors == 0 && nBPP <= 8) {
		nColors = (1 << nBPP);
	}
	DWORD dwBytesPerRow = (((src_lpbi->biWidth * nBPP) + 31) / 32) * 4;
	if (src_lpbi->biCompression == BI_RGB) {
		// uncompressed bitmap, image size might not be set
		src_lpbi->biSizeImage = dwBytesPerRow * src_lpbi->biHeight;
	}

	int size = sizeof(BITMAPINFOHEADER)
			+ sizeof(RGBQUAD) * nColors
			+ src_lpbi->biSizeImage;;
	//if previously pointing to another image, free it.
	HGLOBAL cpy_hdib = ::GlobalAlloc(GMEM_MOVEABLE, size);
	LPBITMAPINFOHEADER cpy_lpbi = (LPBITMAPINFOHEADER)GlobalLock(cpy_hdib);
	//copy the bitmap.
	memcpy(cpy_lpbi, src_lpbi, size);
	GlobalUnlock(cpy_hdib);
	GlobalUnlock(src_hdib);

	return cpy_hdib;
}

/*
* @output hist, the int array sized 256 stores histogram.
* @output totalNum, the total number of pixels from which hist is calculated.
*/
void CDibDoc::HistogramSingle(int *hist, int &totalNum)
{
	LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)GlobalLock(leaf_hdib);
    int i;

    /* initialize the histogram buffer hist[]                           */
    for( i=0; i<256; i++ )
		hist[i] = 0;

    totalNum = 0;

	if (lpbi->biBitCount == 24) // this should be true.
	{
		for (int j=0; j<lpbi->biHeight; j++)
		{
			if (StartIndexScan1[j] >= 0)
			{
				for (i=StartIndexScan1[j]; i<=EndIndexScan1[j]; i++)
				{
					if ((ci_config_geometry->blankout_flag
						&& BlankoutStartI[j] >=0
						&& BlankoutStartI[j] <= i
						&& i <= BlankoutEndI[j])||
                  (ci_config_geometry->blankout_flag
						&& BlankoutStartI2[j] >=0
						&& BlankoutStartI2[j] <= i
						&& i <= BlankoutEndI2[j]))
					{
					}
					else
					{
						BYTE intensity = (BYTE)((leafDibRows[j][i].r+leafDibRows[j][i].g+leafDibRows[j][i].b)/3);
						hist[(int)intensity]++;
						totalNum++;
					}
				}//end i loop
			}//end if
			if (StartIndexScan2[j] >= 0)
			{
				for (i=StartIndexScan2[j]; i<=EndIndexScan2[j]; i++)
				{
					if ((ci_config_geometry->blankout_flag
						&& BlankoutStartI[j] >=0
						&& BlankoutStartI[j] <= i
						&& i <= BlankoutEndI[j])||
                  (ci_config_geometry->blankout_flag
						&& BlankoutStartI2[j] >=0
						&& BlankoutStartI2[j] <= i
						&& i <= BlankoutEndI2[j]))
					{
					}
					else
					{
						BYTE intensity = (leafDibRows[j][i].r+leafDibRows[j][i].g+leafDibRows[j][i].b)/3;
						hist[(int)intensity]++;
						totalNum++;
					}
				}//end i loop
			}
		}//end j loop
	}

	GlobalUnlock(leaf_hdib);
}

/**********************************************************************\
    FUNCTION NAME:
	middle

    DESCRIPTION:
	get the pixel value in the middle of the range, the upper and
	lower percentage values have been taken away.
	Also return the minimum and maximum values of the histogram
	(range)

	Input:  histogram hist[],
		total # of pixels in the image
		and percentage not counted.

    RETURNS:
	int: the middle puxel value;
	     or -1 if error.
\***********************************************************************/

int CDibDoc::Middle(int *hist, int total, int cutoffPcntLow, int cutoffPcntHigh,
	int cutoffPcnt, int &minIntensity, int &maxIntensity )
{
    int  i, low = 256, high = -1;
    int sum = 0;

    minIntensity = -1;
    maxIntensity = -1;

    if(total < MINPIXELNUM)
		return -1;

    /* find the lowest pixel value discarding the noisy ones            */
    sum = 0;
    for( i=0; i<256; i++ )
	{
		sum += hist[i];
		if(sum>0 && minIntensity==-1)
			minIntensity = i;
		if( sum > total*cutoffPcntLow/100 +1) /* - - - - - - - - 09/24/2002 added +1 so it works with 0% - - - - -*/
		{
			low = i;
			break;
		}
    }

    /* find the higher pixel value discarding the noisy ones            */
    sum = 0;
    for( i=255; i>=0; i-- ) {
		sum += hist[i];
		if(sum>0 && maxIntensity==-1)
			maxIntensity = i;
		if( sum > total*cutoffPcntHigh/100 +1)/* - - - - - - - - 09/24/2002 added +1 so it works with 0% - - - - -*/
		{
			high = i;
			break;
		}
    }
    if( high <= low )    /* wrong histogram data, return -1              */
		return -1;

    return (high-low)*cutoffPcnt/100 + low;
}

/**
* return -1 if failed. 0 if succeed.
*/
void CDibDoc::GetLeafDib_MultiThresh()
{
	long int lintensity=0L;
	int cutoff = 0;
	int i, j, k;
	int DISCARD_MIN = 0, DISCARD_MAX = 0,/* - - - - - - was 2% until 09/24/2002 JRG - - - - - - - -*/
		PERCENT_MIDDLE = ci_config_geometry->threshhold_level;

	LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)GlobalLock(leaf_hdib);
	int histgramAll[256];
	int totalNumAll = 0;
	for (k=0; k<256; k++)
		histgramAll[k] = 0;

	//initialize histogram.
	int histograms[MULTI_HIST_ZENITH_DIVISION][MULTI_HIST_AZMUTHAL_DIVISION][256];
	int	totalNums[MULTI_HIST_ZENITH_DIVISION][MULTI_HIST_AZMUTHAL_DIVISION];
	for (j=0; j<MULTI_HIST_ZENITH_DIVISION; j++)
	{
		for (i=0; i<MULTI_HIST_AZMUTHAL_DIVISION; i++)
		{
			cutoffIntensitys[j][i] = -1;
			totalNums[j][i] = 0;
			for (k=0; k<256; k++)
				histograms[j][i][k] = 0;
		}
	}

	//calculate the hitograms.
	for (j=0; j<lpbi->biHeight; j++)
	{
		if (StartIndexScan1[j] >= 0)
		{
			for (i=StartIndexScan1[j]; i<=EndIndexScan1[j]; i++)
			{
				if ((ci_config_geometry->blankout_flag
					&& BlankoutStartI[j] >=0
					&& BlankoutStartI[j] <= i
					&& i <= BlankoutEndI[j])||
                  (ci_config_geometry->blankout_flag
						&& BlankoutStartI2[j] >=0
						&& BlankoutStartI2[j] <= i
						&& i <= BlankoutEndI2[j]))
				{
					//blankout region, do nothing.
				}
				else
				{
					BYTE intensity = (leafDibRows[j][i].r+leafDibRows[j][i].g+leafDibRows[j][i].b)/3;
					int zenithIndex = -1, azmuthalIndex = -1;
					GetDivisionRegion_MultiThresh(i, j, azmuthalIndex, zenithIndex);
					histograms[zenithIndex][azmuthalIndex][intensity]++;
					totalNums[zenithIndex][azmuthalIndex]++;
				}
			}//end i loop
		}//end if
		if (StartIndexScan2[j] >= 0)
		{
			for (i=StartIndexScan2[j]; i<=EndIndexScan2[j]; i++)
			{
				if ((ci_config_geometry->blankout_flag
					&& BlankoutStartI[j] >=0
					&& BlankoutStartI[j] <= i
					&& i <= BlankoutEndI[j])||
                  (ci_config_geometry->blankout_flag
						&& BlankoutStartI2[j] >=0
						&& BlankoutStartI2[j] <= i
						&& i <= BlankoutEndI2[j]))
				{
					//blankout region, do nothing.
				}
				else
				{
					BYTE intensity = (leafDibRows[j][i].r+leafDibRows[j][i].g+leafDibRows[j][i].b)/3;
					int zenithIndex = -1, azmuthalIndex = -1;
					GetDivisionRegion_MultiThresh(i, j, azmuthalIndex, zenithIndex);
					histograms[zenithIndex][azmuthalIndex][intensity]++;
					totalNums[zenithIndex][azmuthalIndex]++;
				}
			}//end i loop
		}//end if
	}

	/*
	* get global histogram.
	*/
	HistogramSingle(histgramAll, totalNumAll);
////////////////////////////////////////////////////////******************************************************&&&&&&*&*&*&*
////////////////////////////////////////////////////////*******************************************************
	for (k=0; k<256; k++)
	{
		lintensity=lintensity+k*histgramAll[k];
	}
//	if(totalNumAll)PAR=(double)lintensity/(double)totalNumAll*ci_config_geometry->PARzerocal;
//	else PAR=(double)0.0;


	int minIntensity, maxIntensity;
	int globalCutoff = Middle(histgramAll, totalNumAll, DISCARD_MIN, DISCARD_MAX, PERCENT_MIDDLE, minIntensity, maxIntensity);
	if (globalCutoff == -1)
		return;

	int intensityRangeAll = maxIntensity - minIntensity;

	/*
	* get cutoff for each histogram.
	*/
	for (j=MULTI_HIST_ZENITH_DIVISION-1; j>=0; j--)
	{
		for (i=0; i<MULTI_HIST_AZMUTHAL_DIVISION; i++)
		{
			//cout << "totalNums[" << j << "][" << i << "]\t" << totalNums[j][i] << endl;
			int tempMin = -1, tempMax = -1;

			int new_thrhd = Middle(histograms[j][i], totalNums[j][i], DISCARD_MIN, DISCARD_MAX, PERCENT_MIDDLE, tempMin, tempMax);

			if( new_thrhd == -1 )
				cutoffIntensitys[j][i] = globalCutoff;
			else if( tempMax-tempMin < intensityRangeAll/2 )
			{
				/* then this area is uniform, treat it as all leaf or all not leaf */
/*				if( new_thrhd < globalCutoff )
					cutoffIntensitys[j][i] = tempMax;
				else if (j < MULTI_HIST_ZENITH_DIVISION-1
						&& tempMax < cutoffIntensitys[j+1][i])
					cutoffIntensitys[j][i] = tempMax;
				else
					cutoffIntensitys[j][i] = tempMin;
 - - - - - changed 9/24/2002 by JRG to get higher LAI values on blurry images- - - - - - - -*/
				cutoffIntensitys[j][i] = globalCutoff;	
			}
			else
				cutoffIntensitys[j][i] = new_thrhd;
		}
	}

	/*
	* initialize the cutoff values on the four corners of a region.
	*/
	THRESH_RECT cutoffRects[MULTI_HIST_ZENITH_DIVISION][MULTI_HIST_AZMUTHAL_DIVISION];
	//center of the circles is the average.
	int cutoff_center = 0;
	for (i=0; i<MULTI_HIST_AZMUTHAL_DIVISION; i++)
		cutoff_center += cutoffIntensitys[0][i];
	cutoff_center /= MULTI_HIST_AZMUTHAL_DIVISION;
	int cutoff_out = 0;
	for (i=0; i<MULTI_HIST_AZMUTHAL_DIVISION; i++)
		cutoff_out += cutoffIntensitys[MULTI_HIST_ZENITH_DIVISION-1][i];
	cutoff_out /= MULTI_HIST_AZMUTHAL_DIVISION;

	int outer_r = ci_config_image->radius * ci_config_geometry->ending_zenith_angle * 2 
				/ ci_config_geometry->view_range;
	int inner_r = ci_config_image->radius * ci_config_geometry->starting_zenith_angle * 2
				/ ci_config_geometry->view_range;
	float dr = ((float)(outer_r - inner_r))/MULTI_HIST_ZENITH_DIVISION;
	int start_a = 0;
	int range = 360;
	if (ci_config_geometry->blankout_flag)
	{
		start_a = ci_config_geometry->blankout2;
		if (ci_config_geometry->blankout1 < ci_config_geometry->blankout2)
			range = 360 - (ci_config_geometry->blankout2 - ci_config_geometry->blankout1);
		else
			range = ci_config_geometry->blankout1 - ci_config_geometry->blankout2;
	}

	float da = ((float)range)/MULTI_HIST_AZMUTHAL_DIVISION;
	int end_a = start_a + range;
	//init the start/end radius and start/end angle.
	for (j=0; j<MULTI_HIST_ZENITH_DIVISION; j++)
	{
		for (i=0; i<MULTI_HIST_AZMUTHAL_DIVISION; i++)
		{
			cutoffRects[j][i].r0 = (int) (inner_r + j * dr);
			cutoffRects[j][i].r1 = (int) (inner_r + (j+1) * dr);
			cutoffRects[j][i].a0 = (int) (start_a + i * da);
			cutoffRects[j][i].a1 = (int) (start_a + (i+1) * da);
		}
	}
	/*
	for (j=0; j<MULTI_HIST_ZENITH_DIVISION; j++)
	{
		for (i=0; i<MULTI_HIST_AZMUTHAL_DIVISION; i++)
		{
			cout << "(" << j << "," << i << "):\t"
				<< "r0 = " << cutoffRects[j][i].r0
				<< ",\tr1 = " << cutoffRects[j][i].r1
				<< ",\ta0 = " << cutoffRects[j][i].a0
				<< ",\ta1 = " << cutoffRects[j][i].a1
				<< endl;
		}
	}
	*/
	int i0, i1, j0, j1;
	//inner and outer most ring.
	for (i=0; i<MULTI_HIST_AZMUTHAL_DIVISION; i++)
	{
		i0 = i;
		j = MULTI_HIST_ZENITH_DIVISION-1;
		i1 = i-1;
		if (i1 < 0) i1 = MULTI_HIST_AZMUTHAL_DIVISION-1;
		cutoffRects[0][i].f00
			= (int)((cutoff_center * 2 * (dr/2)
					 + inner_r * (cutoffIntensitys[0][i0] + cutoffIntensitys[0][i1])
				    )/(dr/2 + inner_r)/2);
		cutoffRects[0][i].f10
			= (cutoffIntensitys[1][i0] + cutoffIntensitys[1][i1] 
			 + cutoffIntensitys[0][i0] + cutoffIntensitys[0][i1])/4;
		cutoffRects[j][i].f00
			= (cutoffIntensitys[j-1][i0] + cutoffIntensitys[j-1][i1]
			 + cutoffIntensitys[j][i0] + cutoffIntensitys[j][i1])/4;
		cutoffRects[j][i].f10
			= (cutoff_out * 2 + cutoffIntensitys[j][i0] + cutoffIntensitys[j][i1])/4;

		i1 = i+1;
		if (i1 == MULTI_HIST_AZMUTHAL_DIVISION) i1 = 0;
		cutoffRects[0][i].f01
			= (int)((cutoff_center * 2 * (dr/2)
				     + inner_r * (cutoffIntensitys[0][i0] + cutoffIntensitys[0][i1])
					)/(dr/2 + inner_r)/2);
		cutoffRects[0][i].f11
			= (cutoffIntensitys[1][i0] + cutoffIntensitys[1][i1]
			 + cutoffIntensitys[0][i0] + cutoffIntensitys[0][i1])/4;
		cutoffRects[j][i].f01
			= (cutoffIntensitys[j-1][i0] + cutoffIntensitys[j-1][i1]
			 + cutoffIntensitys[j][i0] + cutoffIntensitys[j][i1])/4;
		cutoffRects[j][i].f11
			= (cutoff_out * 2 + cutoffIntensitys[j][i0] + cutoffIntensitys[j][i1])/4;
	}
	//all the other rings
	for (j=1; j<MULTI_HIST_ZENITH_DIVISION-1; j++)
	{
		for (i=0; i<MULTI_HIST_AZMUTHAL_DIVISION; i++)
		{
			i0 = i;
			j0 = j;
			i1 = i-1;
			if (i1 < 0) i1 = MULTI_HIST_AZMUTHAL_DIVISION-1;
			j1 = j-1;
			cutoffRects[j][i].f00
				= (cutoffIntensitys[j0][i0] + cutoffIntensitys[j0][i1]
				 + cutoffIntensitys[j1][i0] + cutoffIntensitys[j1][i1])/4;
			j1 = j+1;
			cutoffRects[j][i].f10
				= (cutoffIntensitys[j0][i0] + cutoffIntensitys[j0][i1]
				 + cutoffIntensitys[j1][i0] + cutoffIntensitys[j1][i1])/4;
			j1 = j-1;
			i1 = i+1;
			if (i1 == MULTI_HIST_AZMUTHAL_DIVISION) i1 = 0;
			cutoffRects[j][i].f01
				= (cutoffIntensitys[j0][i0] + cutoffIntensitys[j0][i1]
				 + cutoffIntensitys[j1][i0] + cutoffIntensitys[j1][i1])/4;
			j1 = j+1;
			cutoffRects[j][i].f11
				= (cutoffIntensitys[j0][i0] + cutoffIntensitys[j0][i1]
				 + cutoffIntensitys[j1][i0] + cutoffIntensitys[j1][i1])/4;
		}
	}

	/*
	* using the histograms to get leaf distribution.
	*/
	for (j=0; j<lpbi->biHeight; j++)
	{
		if (StartIndexScan1[j] >= 0)
		{
			for (i=StartIndexScan1[j]; i<=EndIndexScan1[j]; i++)
			{
				if ((ci_config_geometry->blankout_flag
					&& BlankoutStartI[j] >=0
					&& BlankoutStartI[j] <= i
					&& i <= BlankoutEndI[j])||
                  (ci_config_geometry->blankout_flag
						&& BlankoutStartI2[j] >=0
						&& BlankoutStartI2[j] <= i
						&& i <= BlankoutEndI2[j]))
				{
					SetBlankoutRegion(i, j);
				}
				else
				{
					int intensity = (leafDibRows[j][i].r+leafDibRows[j][i].g+leafDibRows[j][i].b)/3;
					int zenithIndex = -1, azmuthalIndex = -1;
					GetDivisionRegion_MultiThresh(i, j, azmuthalIndex, zenithIndex);
					int cutoffIntensity = GetCutoffIntensity_MultiThresh(&cutoffRects[zenithIndex][azmuthalIndex], i, j);
					if (minIntensity <= intensity
						&& intensity < cutoffIntensity
						&& intensity <= maxIntensity)
						SetLeaf(i, j);
					else
						SetNotLeaf(i, j);
				}
			}// end i loop
		}//end if
		if (StartIndexScan2[j] >= 0)
		{
			for (i=StartIndexScan2[j]; i<=EndIndexScan2[j]; i++)
			{
				if ((ci_config_geometry->blankout_flag
					&& BlankoutStartI[j] >=0
					&& BlankoutStartI[j] <= i
					&& i <= BlankoutEndI[j])||
                  (ci_config_geometry->blankout_flag
						&& BlankoutStartI2[j] >=0
						&& BlankoutStartI2[j] <= i
						&& i <= BlankoutEndI2[j]))
				{
					SetBlankoutRegion(i, j);
				}
				else
				{
					int intensity = (leafDibRows[j][i].r+leafDibRows[j][i].g+leafDibRows[j][i].b)/3;
					int zenithIndex = -1, azmuthalIndex = -1;
					GetDivisionRegion_MultiThresh(i, j, azmuthalIndex, zenithIndex);
					int cutoffIntensity = GetCutoffIntensity_MultiThresh(&cutoffRects[zenithIndex][azmuthalIndex], i, j);
					if (minIntensity <= intensity
						&& intensity < cutoffIntensity
						&& intensity <= maxIntensity)
						SetLeaf(i, j);
					else
						SetNotLeaf(i, j);
				}
			}// end i loop
		}//end if
	}

}

/*
* get the cutoffIntensity for the pixel.
*/
int CDibDoc::GetCutoffIntensity_MultiThresh(THRESH_RECT *tr, int i, int j)
{
	int r = Radius(i, j);
	int a = Angle(i, j);
	if (a < tr->a0) a +=360;

	if (r > tr->r1)
		r = tr->r1;
	if (r < tr->r0)
		r = tr->r0;
	if (a > tr->a1)
		a = tr->a1;
	if (a < tr->a0)
		a = tr->a0;

	if (r > tr->r1 || r < tr->r0)
	{
		cout << "Warning! radius out of range. pixel(" << j << "," << i << "): "
			 << "r = " << r << ", r0 = " << tr->r0 << ", r1 = " << tr->r1
			 << endl;
		return (tr->f00 + tr->f01 +tr->f10 + tr->f11)/4;
	}
	else if (a > tr->a1 || a < tr->a0)
	{
		cout << "Warning! angle out of range. pixel(" << j << "," << i << "): "
			 << "a = " << a << ", a0 = " << tr->a0 << ", a1 = " << tr->a1
			 << endl;
		return (tr->f00 + tr->f01 +tr->f10 + tr->f11)/4;
	}
	else
	{
      if(((tr->r1 - tr->r0)*(tr->a1 - tr->a0))!=0)
		return ((tr->f11*(r - tr->r0) + tr->f01*(tr->r1 - r))*(a - tr->a0)
			   +(tr->f10*(r - tr->r0) + tr->f00*(tr->r1 - r))*(tr->a1 - a))
			  /((tr->r1 - tr->r0)*(tr->a1 - tr->a0));
      else return 127;
	}
}

/*
* Get the azmuthal and zenith indices for a pixel.
*/
void CDibDoc::GetDivisionRegion_MultiThresh(int i, int j, int& azmuthalIndex, int& zenithIndex)
{
	int outer_r = ci_config_image->radius * ci_config_geometry->ending_zenith_angle * 2 
				/ ci_config_geometry->view_range;
	int inner_r = ci_config_image->radius * ci_config_geometry->starting_zenith_angle * 2
				/ ci_config_geometry->view_range;
	float dr = ((float)(outer_r - inner_r))/MULTI_HIST_ZENITH_DIVISION;
	int r = Radius(i, j);
	if (r <= inner_r)
		zenithIndex = 0;
	else if (r >= outer_r)
		zenithIndex = MULTI_HIST_ZENITH_DIVISION - 1;
	else	
		zenithIndex = (int)((r - inner_r)/dr);
	
	
	int az = Angle(i, j);
	int range = 360;
	if (ci_config_geometry->blankout_flag)
	{
		if ((BlankoutStartI[j] >=0
			&& BlankoutStartI[j] <= i
			&& i <= BlankoutEndI[j])||(
                  BlankoutStartI2[j] >=0
						&& BlankoutStartI2[j] <= i
						&& i <= BlankoutEndI2[j]))
		{
			//the point is in blankout region.
			azmuthalIndex = -1;
			return;
		}
		else
		{
			az = az - ci_config_geometry->blankout2;
			if (az < 0)
				az += 360;
			if (ci_config_geometry->blankout1 < ci_config_geometry->blankout2)
				range = 360 - (ci_config_geometry->blankout2 - ci_config_geometry->blankout1);
			else
				range = ci_config_geometry->blankout1 - ci_config_geometry->blankout2;
		}
	}

	float da = ((float)range)/MULTI_HIST_AZMUTHAL_DIVISION;
	if (az <= 0)
		azmuthalIndex = 0;
	else if (az >= range)
		azmuthalIndex = MULTI_HIST_AZMUTHAL_DIVISION - 1;
	else
		azmuthalIndex = (int) (az / da);
}

void CDibDoc::GetLeafDib()
{
	if (state != CI_SHOWCROPPEDIMAGE && state != CI_SHOWLEAFIMAGE)
		return;

	//if previously pointing to another leaf image, free it.
	if (leaf_hdib && leaf_hdib!=cropped_hdib && leaf_hdib!=original_hdib) {
		GlobalFree(leaf_hdib);
		leaf_hdib = NULL;
	}
	leaf_hdib = copyBitMap(cropped_hdib);

	LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)GlobalLock(leaf_hdib);

	//now begin finding leaf area.
	unsigned char* ptr = (unsigned char*) lpbi;
	int		nBPP = lpbi->biBitCount;
	int		nColors = (int)lpbi->biClrUsed;
	// figure out actual size of color table
	if (nColors == 0 && nBPP <= 8) {
		nColors = (1 << nBPP);
	}
	DWORD dwBytesPerRow = (((lpbi->biWidth * nBPP) + 31) / 32) * 4;
	//pointing to the image data.
	ptr = ptr + sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD) * nColors;
	for (int i=0; i<lpbi->biHeight; i++)
		leafDibRows[i] = (COLORPOINT *) (ptr + i * dwBytesPerRow);

	//GetLeafDib_Peak();
	GetLeafDib_MultiThresh();

	//don't draw geometry divisions.
	SetDrawGeoDiv(FALSE);
	state = CI_SHOWLEAFIMAGE;

}

void CDibDoc::GetLeafDib_Peak()
{
	int cutoffIntensity = GetLeafCutoffIntensity_Peak();
	int minIntensity = 0;
	int maxIntensity = 0x7FFFFFFF;

	LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)GlobalLock(leaf_hdib);
	if (lpbi->biBitCount == 24) // this should be true.
	{
		for (int j=0; j<lpbi->biHeight; j++)
		{
			if (StartIndexScan1[j] >= 0)
			{
				for (int i=StartIndexScan1[j]; i<=EndIndexScan1[j]; i++)
				{
					if ((ci_config_geometry->blankout_flag
						&& BlankoutStartI[j] >=0
						&& BlankoutStartI[j] <= i
						&& i <= BlankoutEndI[j])||
                  (ci_config_geometry->blankout_flag
						&& BlankoutStartI2[j] >=0
						&& BlankoutStartI2[j] <= i
						&& i <= BlankoutEndI2[j]))
					{
					}
					else
					{
						int intensity = (leafDibRows[j][i].r+leafDibRows[j][i].g+leafDibRows[j][i].b)/3;
						if (minIntensity < intensity && intensity < cutoffIntensity && intensity < maxIntensity)
							SetLeaf(i, j);
						else
							SetNotLeaf(i, j);
					}
				}// end i loop
			}//end if
			if (StartIndexScan2[j] >= 0)
			{
				for (int i=StartIndexScan2[j]; i<=EndIndexScan2[j]; i++)
				{
					if ((ci_config_geometry->blankout_flag
						&& BlankoutStartI[j] >=0
						&& BlankoutStartI[j] <= i
						&& i <= BlankoutEndI[j])||
                  (ci_config_geometry->blankout_flag
						&& BlankoutStartI2[j] >=0
						&& BlankoutStartI2[j] <= i
						&& i <= BlankoutEndI2[j]))
					{
					}
					else
					{
						int intensity = (leafDibRows[j][i].r+leafDibRows[j][i].g+leafDibRows[j][i].b)/3;
						if (minIntensity < intensity && intensity < cutoffIntensity && intensity < maxIntensity)
							SetLeaf(i, j);
						else
							SetNotLeaf(i, j);
					}
				}// end i loop
			}//end if
		}// end j loop
	}
}

int CDibDoc::GetLeafCutoffIntensity_Peak()
{
	int cutoff = 0;

	LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)GlobalLock(leaf_hdib);

	//HISTOGRAM	histograms[MAX_NUM_ZENITH_DIVISION][MAX_NUM_ZENITH_DIVISION];
	HISTOGRAM	histogram_all;

	//remember in bitmap, the cordinates start at lower left corner, while
	//in device cordinates, it starts at upper left corner.
	if (lpbi->biBitCount == 24) // this should be true.
	{
		LONG avg_r = 0;
		LONG avg_g = 0;
		LONG avg_b = 0;
		int count = 0;

		for (int i=0;i<256;i++)
		{
			histogram_all.dist_r[i] = 0;
			histogram_all.dist_g[i] = 0;
			histogram_all.dist_b[i] = 0;
		}

		/*for (j=0; j<lpbi->biHeight; j++)
			for (i=0; i<lpbi->biWidth; i++)
				leafDibRows[j][i].g &= 0xFE;
		*/
		for (int j=0; j<lpbi->biHeight; j++)
		{
			if (StartIndexScan1[j] >= 0)
			{
				for (int i=StartIndexScan1[j]; i<=EndIndexScan1[j]; i++)
				{
					if (BlankoutStartI[j] == -1 ||
						!((BlankoutStartI[j] <= i && BlankoutEndI[j] >= i)||
                  ( BlankoutStartI2[j] >=0
						&& BlankoutStartI2[j] <= i
						&& i <= BlankoutEndI2[j])))
					{
						avg_r += leafDibRows[j][i].r;
						avg_g += leafDibRows[j][i].g;
						avg_b += leafDibRows[j][i].b;
						count ++;
						//
						histogram_all.dist_r[leafDibRows[j][i].r] += 1;
						histogram_all.dist_g[leafDibRows[j][i].g] += 1;
						histogram_all.dist_b[leafDibRows[j][i].b] += 1;
						//
					}
				}// end i loop
			}//end if
			if (StartIndexScan2[j] >= 0)
			{
				for (i=StartIndexScan2[j]; i<=EndIndexScan2[j]; i++)
				{
					if (BlankoutStartI[j] == -1 ||
						!((BlankoutStartI[j] <= i && BlankoutEndI[j] >= i)||
                  ( BlankoutStartI2[j] >=0
						&& BlankoutStartI2[j] <= i
						&& i <= BlankoutEndI2[j])))
					{
						avg_r += leafDibRows[j][i].r;
						avg_g += leafDibRows[j][i].g;
						avg_b += leafDibRows[j][i].b;
						count ++;
						//
						histogram_all.dist_r[leafDibRows[j][i].r] += 1;
						histogram_all.dist_g[leafDibRows[j][i].g] += 1;
						histogram_all.dist_b[leafDibRows[j][i].b] += 1;
						//
					}
				}// end i loop
			}//end if
		}// end j loop
		avg_r /= count;
		avg_g /= count;
		avg_b /= count;
		//
		int intensity[256];
		for (i=0; i<256; i++)
			intensity[i] = (histogram_all.dist_r[i] + histogram_all.dist_g[i] + histogram_all.dist_b[i]) / 3;
		//ShowHistogram("red:", histogram_all.dist_r);
		//ShowHistogram("green:", histogram_all.dist_g);
		//ShowHistogram("blue:", histogram_all.dist_b);
		//ShowHistogram("total:", intensity);
		GetLeafIntensity(intensity, cutoff);
		//
	}
	else
	{
		//
	}
	GlobalUnlock(leaf_hdib);

	return cutoff;
}

//Input: a histogram hist sized sizeH, array of PULSE's sized sizeP
//Output:
//    pulses
//        positions and values of maximums, the widths of the maximums,
//        and number of pixels falling in the pulse.
//    numOfPulse
//        number of maximums.
void CDibDoc::GetMaximums(int *hist, int sizeH, PULSE *pulses, int sizeP, int& numOfPulses)
{
	numOfPulses = 0;
	BOOL	hasPulse = TRUE;
	int i, j;
	int max_value, max_index;
	//initialize pulses.
	for (i=0; i<sizeP; i++)
	{
		pulses[i].index_left_half = -1;
		pulses[i].index_max = -1;
		pulses[i].index_right_half = -1;
		pulses[i].max_value = -1;
		pulses[i].count = -1;
	}

	while( hasPulse && numOfPulses<sizeP)
	{
		//first to find the max value in the histogram that is not in other pulses already
		//found.
		max_value = -1;
		for (i=0; i<255; i++)
		{
			//assume this point hasn't been included in other pulses.
			BOOL isNew = TRUE;
			for (j=0; j<numOfPulses; j++)
			{
				//check whether point i has been included in other pulses.
				if (pulses[j].index_left_half <= i && i <= pulses[j].index_right_half)
				{
					isNew = FALSE;
					i = pulses[j].index_right_half;
					break;
				}
			}
			if (isNew)
			{
				if (max_value < hist[i])
				{
					if (i==0 || i==(sizeH-1) || (hist[i-1]<hist[i] && hist[i]>=hist[i+1]) )
					{
						max_value = hist[i];
						max_index = i;
					}
				}
			}
		}

		//if there is a max, then calculate its parameters.
		if (max_value > 0)
		{
			pulses[numOfPulses].max_value = max_value;
			pulses[numOfPulses].index_max = max_index;
			//find the index_left_half.
			for (i=max_index; i>0; i--)
			{
				//assume this point hasn't been included in other pulses.
				BOOL isNew = TRUE;
				for (j=0; j<numOfPulses; j++)
				{
					//check whether point i has been included in other pulses.
					if (pulses[j].index_left_half <= i && i <= pulses[j].index_right_half)
					{
						isNew = FALSE;
						break;
					}
				}

				if(!isNew || hist[i] <= max_value/2 || i==0 || i==(sizeH-1) ||
					(hist[i-1]>hist[i]&&hist[i]<hist[i+1]&&hist[i]<=max_value*3/4) )
				{
					pulses[numOfPulses].index_left_half = i;
					break;
				}
			}
			//find the index_right_half.
			for (i=max_index; i<sizeH; i++)
			{
				//assume this point hasn't been included in other pulses.
				BOOL isNew = TRUE;
				for (j=0; j<numOfPulses; j++)
				{
					//check whether point i has been included in other pulses.
					if (pulses[j].index_left_half <= i && i <= pulses[j].index_right_half)
					{
						isNew = FALSE;
						break;
					}
				}

				if(!isNew || hist[i] <= max_value/2 || i==0 || i==(sizeH-1) ||
					(hist[i-1]>hist[i]&&hist[i]<hist[i+1]&&hist[i]<=max_value*3/4) )
				{
					pulses[numOfPulses].index_right_half = i;
					break;
				}
			}

			//get the counts;
			pulses[numOfPulses].count = 0;
			for (j=pulses[numOfPulses].index_left_half; j<=pulses[numOfPulses].index_right_half; j++)
				pulses[numOfPulses].count += hist[j];
			numOfPulses++;
		}
		else
			hasPulse = FALSE;
	}
}

//get the index of the minimum value in the array between i0 and i1.
int CDibDoc::GetIndexOfMinimum(int *smoothed_intensity, int i0, int i1)
{
	if (i0==i1)
		return i0;
	if (i0 > i1)
	{
		int tmp = i0;
		i0 = i1;
		i1 = tmp;
	}

	int I0 = smoothed_intensity[i0], I1 =  smoothed_intensity[i1];
	int middle;
	if (I0 > 100 && I1 > 100)
	{
		if (I0 > 4*I1 || I1 > 4*I0)
		{
			middle = (i0 * I0 + i1 * I1)/(I0 + I1);
		}
		else
		{
			//find the minimum between i0 and i1;
			int min = smoothed_intensity[i0];
			int index_min = i0;
			for (int i=i0+1; i<=i1; i++)
				if (smoothed_intensity[i] < min)
				{
					min = smoothed_intensity[i];
					index_min = i;
				}

			if (index_min == i0 || index_min == i1)
				return (i0+i1)/2;
			else
				return index_min;
		}
	}
	else
	{
		middle = (i0 + i1)/2;
	}

	if (smoothed_intensity[middle-1] >= smoothed_intensity[middle])
		while (middle<i1 && smoothed_intensity[middle]>smoothed_intensity[middle+1])
			middle++;
	else
		while (middle>i0 && smoothed_intensity[middle]>smoothed_intensity[middle-1])
			middle--;

	if (middle==i1 || middle==i0)
		return (i0+i1)/2;
	else
		return middle;
}

void CDibDoc::GetLeafIntensity(int *hist, int &cutoff)
{
	int i, j;
	int min = 0, max = 0;

	//smooth the histogram by averaging the intensity.
	int smoothed_intensity[256];
	int smoothWeight[2*SMOOTH_REGION_WIDTH + 1];
	for (i=0; i<SMOOTH_REGION_WIDTH+1; i++)
		smoothWeight[i] = i+1;
	for (i=SMOOTH_REGION_WIDTH+1; i<2*SMOOTH_REGION_WIDTH+1; i++)
		smoothWeight[i] = 2 * SMOOTH_REGION_WIDTH + 1 - i;
	int weight;

	for (i=0; i<256; i++)
	{
		smoothed_intensity[i] = 0;
		weight = 0;
		min = - SMOOTH_REGION_WIDTH;
		max = SMOOTH_REGION_WIDTH;
		if (min+i < 0) min = -i;
		if (max+i > 255) max = 255-i;
		for (j=i+min; j<=i+max; j++)
		{
			smoothed_intensity[i] += (hist[j] * smoothWeight[j-i+SMOOTH_REGION_WIDTH]);
			weight += smoothWeight[j-i+SMOOTH_REGION_WIDTH];
		}
		smoothed_intensity[i] /= weight;
	}

	//find out the cutoff.
	BOOL found = FALSE;
	/*
	int threshhood = 8;
	BOOL maxFound = FALSE;
	BOOL minFound = FALSE;
	while (!found && threshhood>=4)
	{
		threshhood /=2;
		min = smoothed_intensity[0];
		max = smoothed_intensity[0];
		for (i=1; i<255; i++)
		{
			if (smoothed_intensity[i] > max)
			{
				max = smoothed_intensity[i];
				min = max;
				if (max >= smoothed_intensity[i+1])
					maxFound = TRUE;
				else
					maxFound = FALSE;
			}
			if (maxFound)
			{
				if (smoothed_intensity[i] < min)
				{
					min = smoothed_intensity[i];
					if (min <= smoothed_intensity[i+1])
					{
						minFound = TRUE;
						//cutoff = i;
						if (min < max/threshhood)
						{
							found = TRUE;
							cutoff = i;
							break;
						}
					}
				}
				if (minFound && smoothed_intensity[i] > 2*min)
					break; //break out of for loop.
			}
		}
	}
	//if not found return the average value.
	if (!found)
	{
		long total = 0;
		long count = 0;
		for (i=0; i<256; i++)
		{
			total += (i * hist[i]);
			count += hist[i];
		}
		cutoff = total / count;
	}
	*/
	PULSE pulses[4];
	int numOfPulses;
	int i0=0, i1=255;
	GetMaximums(smoothed_intensity, 256, pulses, 4, numOfPulses);
	if (numOfPulses>1)
	{
		if (pulses[0].index_max <= 127)
		{
			i0 = pulses[0].index_max;
			i = 1;
			while (i<4 && pulses[i].index_max<=127) i++;
			if (i==4)
				i1 = 255;
			else
				i1 = pulses[i].index_max;
		}
		else
		{
			i1 = pulses[0].index_max;
			i = 1;
			while (i<4 && pulses[i].index_max>127) i++;
			if (i==4)
				i0 = 0;
			else
				i0 = pulses[i].index_max;
		}
	}
	else if (numOfPulses==1)
	{
		if (pulses[0].index_max >= 127)
		{
			i1 = pulses[0].index_max;
			i0 = 0;
		}
		else
		{
			i0 = pulses[0].index_max;
			i1 = 255;
		}
	}

	cutoff = GetIndexOfMinimum(smoothed_intensity, i0, i1);

	//ShowHistogram("smoothed:", smoothed_intensity);

}

void CDibDoc::ShowHistogram(char *title, int *hist)
{
	int i, j;
	CIResultDialog dlg;
	char result[10240];
	char line[256];
	memset(result, 0, 10240);
	memset(line, 0, 256);
	strcat(result, title);
	strcat(result, "\r\n");

	long avg = 0;
	long count = 0;
	for (i=0; i<256; i++)
	{
		for (j=0; j<hist[i]/10; j++)
			strcat(result, "*");
		sprintf(line, "%d\r\n", i);
		strcat(result, line);
		avg += (i * hist[i]);
		count += hist[i];
	}
	avg /= count;
	sprintf(line, "avg: %d", avg);
	strcat(result, line);
	int templen = strlen(result);
	dlg.m_ResultText = CString(result);
	dlg.DoModal();
	dlg.DestroyWindow();
}

BOOL CDibDoc::IsCropable()
{
	return (state==CI_SHOWCROPPEDIMAGE && m_drawGeometryDivision);
}

//if the pixel is a leaf or not
//param x and y are in bitmap cordinates.
BOOL CDibDoc::IsLeaf(int i, int j)
{
	/*
	if (leafDibRows[y][x].r==0 && leafDibRows[y][x].g==0 && leafDibRows[y][x].b==0)
		return TRUE;
	else
		return FALSE;
	*/
	/*if (leafDibRows[j][i].g == 0xff)
		return TRUE;
	else
		return FALSE;
	*/
	if (leafDibRows[j][i].g == 0xff && leafDibRows[j][i].r < 0xd0) //want to eliminate white
		return TRUE;
	else
		return FALSE;
}

//set the leaf pixel to have special color.
//param x and y are in bitmap cordinates.
void CDibDoc::SetLeaf(int i, int j)
{
	leafDibRows[j][i].g = 0xff;
}

//set the leaf pixel to have special color.
//param x and y are in bitmap cordinates.
void CDibDoc::SetNotLeaf(int i, int j)
{
	// for non-leaf region, maintain the original color/brightness
	/*
	leafDibRows[j][i].r /= 2;
	leafDibRows[j][i].g /= 2;
	leafDibRows[j][i].b /= 2;
	*/
}

//set the leaf pixel to have special color.
//param x and y are in bitmap cordinates.
void CDibDoc::SetBlankoutRegion(int i, int j)
{
	// for blankout region, darken the color by 4
	leafDibRows[j][i].r >>= 1;
	leafDibRows[j][i].g >>= 1;
	leafDibRows[j][i].b >>= 1;
}

//return if the pixel is in blankout region or not.
//param x and y are in image cordinates, counting from lower-left corner.
BOOL CDibDoc::IsInBlankoutRegion(int i, int j)
{

	if (ci_config_geometry->blankout_flag)
	{
		int angle = Angle(i, j);
		if (ci_config_geometry->blankout1 < ci_config_geometry->blankout2)
		{
			if (angle >= ci_config_geometry->blankout1 &&
				angle <= ci_config_geometry->blankout2)
				return TRUE;
			else
				return FALSE;
		}
		else
		{
			if (angle >= ci_config_geometry->blankout1 ||
				angle <= (ci_config_geometry->blankout2 ))
		/*	if (angle >= ci_config_geometry->blankout1 &&
				angle <= (ci_config_geometry->blankout2 + 360))*/
				return TRUE;
			if (angle <= ci_config_geometry->blankout2)
				return TRUE;

			return FALSE;
		}
	}
	else
		return FALSE;
}

//return if the azmuthal region overlaps with blankout region or not.
//param i is the azmuthal region index.
BOOL CDibDoc::IsInBlankoutRegion(int i)
{
	if (i < 0 || i >= ci_config_geometry->num_of_azmuthal_division)
		return FALSE;

	if (ci_config_geometry->blankout_flag)
	{
		int minAngle = i * 360 / ci_config_geometry->num_of_azmuthal_division ;
		int maxAngle = (i+1) * 360 / ci_config_geometry->num_of_azmuthal_division ;
		if (ci_config_geometry->blankout1 < ci_config_geometry->blankout2)
		{
			if (minAngle > ci_config_geometry->blankout2 ||
				maxAngle < ci_config_geometry->blankout1)
				return FALSE;
			else
				return TRUE;
		}
		else
		{
			if (minAngle >= ci_config_geometry->blankout2 &&
				maxAngle <= ci_config_geometry->blankout1)
				return FALSE;
			else
				return TRUE;
		}
	}
	else
		return FALSE;
}

void CDibDoc::Get_trans_coeff()
{
	LONG area[MAX_NUM_ZENITH_DIVISION];
	LONG total_area[MAX_NUM_ZENITH_DIVISION];
    int i, j;
    int firstblank=0;
	for( i=0; i<ci_config_geometry->num_of_zenith_division; i++ ) {
		ci_result.tranmission_efficient[i] = 0.;
		total_area[i] = 0;
		area[i] = 0;
    }

	LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)GlobalLock(leaf_hdib);
	for (j=0; j<lpbi->biHeight; j++)
	{
		if (StartIndexScan1[j] >= 0)
		{
			//int y = devCordY(j, ci_config_image->center_y, lpbi->biHeight);
			for (i=StartIndexScan1[j]; i<=EndIndexScan1[j]; i++)
			{
			//	if (BlankoutStartI[j] == -1 ||
			//		!((BlankoutStartI[j] <= i && BlankoutEndI[j] >= i)||
         //         ( BlankoutStartI2[j] >=0
			//			&& BlankoutStartI2[j] <= i
			//			&& i <= BlankoutEndI2[j])))
				{
					int index = RingIndex(i, j);
					if (index >=0)
					{
						total_area[index]++;
						if (IsLeaf(i, j))
							area[index]++;
					}
				}
			}
		}
		if (StartIndexScan2[j] >= 0)
		{
			//int y = devCordY(j, ci_config_image->center_y, lpbi->biHeight);
			for (i=StartIndexScan2[j]; i<=EndIndexScan2[j]; i++)
			{
		//		if (BlankoutStartI[j] == -1 ||
		//			!((BlankoutStartI[j] <= i && BlankoutEndI[j] >= i)||
      //            ( BlankoutStartI2[j] >=0
		//				&& BlankoutStartI2[j] <= i
		//				&& i <= BlankoutEndI2[j])))
				{
					int index = RingIndex(i, j);
					if (index >=0)
					{
						total_area[index]++;
						if (IsLeaf(i, j))
							area[index]++;
					}
				}
			}
		}
	}

    /* calculate the transmission coefficient for each rings            */
    for( i=0; i<ci_config_geometry->num_of_zenith_division; i++ ) {
		if( total_area[i] != 0 )
			ci_result.tranmission_efficient[i] = 1. - (double)area[i]/(double)total_area[i];
      else ci_result.tranmission_efficient[i] = -1.;
    }
	GlobalUnlock(leaf_hdib);
}

//find out the azmuthal division index, ranged [0, ci_config_geometry->num_of_azmuthal_division);
//param i and j are in image cordinates, counting from lower-left corner.
//The angle is anti-clockwise starting from east.
int CDibDoc::AngleIndex(int i, int j)
{
      				if (ci_config_geometry->blankout_flag
					&& BlankoutStartI[j] >=0
					&& BlankoutStartI[j] <= i
					&& i <= BlankoutEndI[j])
					return -1;
   				if (ci_config_geometry->blankout_flag
					&& BlankoutStartI[j] >=0
					&& BlankoutStartI[j] <= i
					&& i <= BlankoutEndI[j])
					return -1;

	int angle = Angle(i, j);
	int index = angle * ci_config_geometry->num_of_azmuthal_division / 360;
	if (index >=  ci_config_geometry->num_of_azmuthal_division)
	{
		printf("wrong angle index (%d)!", index);
		exit(1);
	}
	return index;
}

//radius from the center of image.
//param i and j are in image cordinates, counting from lower-left corner.
int CDibDoc::Radius(int i, int j)
{
	int dx = i - ci_config_image->center_x;
	int dy = j - ci_config_image->center_y;

	return (int) ci_sqrt(dx*dx + dy*dy);
}

//angle within [0, 360).
//param i and j are in image cordinates, counting from lower-left corner.
//The angle is anti-clockwise starting from east.
int CDibDoc::Angle(int i, int j)
{
	int dx = i - ci_config_image->center_x;
	int dy = j - ci_config_image->center_y;
   int signflag;

   signflag=0;
   if(dx<0&&dy>0){signflag=1;dx=-dx;}
   if(dx>0&&dy<0){signflag=3;dy=-dy;}
   if(dx<0&&dy<0){signflag=2;}
   

	int angle;
	if(TRUE)
	{
		if (dx != 0 && dy != 0)
		{
			angle = (int)((double)180 * ci_atan((double)dy / dx) / CI_PI);
		//	if (angle == 0 && dy < 0)
			//	angle = -.0000000001;

			if (angle >= 90)
			{
				printf("angle = %d.\n", angle);
			}
			if (angle <= -90)
			{
				printf("angle = %d.\n", angle);
			}
	/*		if (dx > 0 && dy > 0)
			{
				//do nothing.
			}
			else if (dx > 0 && dy < 0)
			{
				//angle was negtive, need to map to (0, 360);
				if (angle < 0 )
					angle += 360;
			}
			if (dx < 0 && dy > 0)
			{
				//angle was negtive, need to map to (0, 360);
				angle += 180;
			}
			if (dx < 0 && dy < 0)
			{
				angle += 180;
			}*/
         if(signflag==1)angle=180-angle;
         if(signflag==2)angle=180+angle;
         if(signflag==3)angle=360-angle;
		}
		else
		{
			if (dx == 0)
			{
				if (dy >= 0)
					angle = 90;
				else
					angle = 270;
			}
			else if (dy ==0)
			{
				if (dx > 0)
					angle = 0;
				else
					angle = 180;
			}
		}
	}
   angle-=90;
   if(angle<0)angle+=360; /* adjustments to angle in 3.1.0.1 to correct for earlier change from W=0 to N=0  Aug 3, 2005*/
	return angle;
}


void CDibDoc::Get_leaf_dist()
{
	LONG area[MAX_NUM_AZMUTHAL_DIVISION];
	LONG total_area[MAX_NUM_AZMUTHAL_DIVISION];
    int i, j;
	for( i=0; i<MAX_NUM_AZMUTHAL_DIVISION; i++ ) {
		ci_result.leaf_distribution[i] = 0.;
		total_area[i] = 0;
		area[i] = 0;
    }

	LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)GlobalLock(leaf_hdib);
	int height = lpbi->biHeight;

	for (j=0; j<height; j++)
	{
		if (StartIndexScan1[j] >= 0)
		{
			for (i=StartIndexScan1[j]; i<=EndIndexScan1[j]; i++)
			{
		//		if (BlankoutStartI[j] == -1 ||
		//			!((BlankoutStartI[j] <= i && BlankoutEndI[j] >= i)||
        //          ( BlankoutStartI2[j] >=0
			//			&& BlankoutStartI2[j] <= i
			//			&& i <= BlankoutEndI2[j])))
				{
					int angleindex = AngleIndex(i, j);
					int ringindex = RingIndex(i, j);
					if (angleindex >= 0 && ringindex >= 0)
					{
						total_area[angleindex]++;
						if (IsLeaf(i, j))
							area[angleindex]++;
					}
				}
			}
		}
		if (StartIndexScan2[j] >= 0)
		{
			for (i=StartIndexScan2[j]; i<=EndIndexScan2[j]; i++)
			{
			//	if (BlankoutStartI[j] == -1 ||
			//		!((BlankoutStartI[j] <= i && BlankoutEndI[j] >= i)||
           //       ( BlankoutStartI2[j] >=0
				//		&& BlankoutStartI2[j] <= i
				//		&& i <= BlankoutEndI2[j])))
				{
					int angleindex = AngleIndex(i, j);
					int ringindex = RingIndex(i, j);
					if (angleindex >= 0 && ringindex >= 0)
					{
						total_area[angleindex]++;
						if (IsLeaf(i, j))
							area[angleindex]++;
					}
				}
			}
		}
	}

    /* calculate the transmission coefficient for each rings            */
    for( i=0; i<ci_config_geometry->num_of_azmuthal_division; i++ ) {
		if( total_area[i] != 0 )
			ci_result.leaf_distribution[i] = (double)area[i]/(double)total_area[i];
		else
			ci_result.leaf_distribution[i] = 0.;
    }
	GlobalUnlock(leaf_hdib);
}

void CDibDoc::Measure()
{
	CCIDPreMsrDialog dlg(this, NULL);
	dlg.UpdateState();
	if (dlg.DoModal() == IDOK)
		dlg.DestroyWindow();
	else
	{
		dlg.DestroyWindow();
		return;
	}


	if (!dlg.blankout_applied)
	{
		::SetCursor(::LoadCursor(NULL, IDC_WAIT));

		GetLeafDib();
		UpdateAllViews(NULL);
	}

	Get_leaf_dist();
	Get_trans_coeff();

	double zenith_ang[MAX_NUM_ZENITH_DIVISION];
	double z[MAX_NUM_ZENITH_DIVISION];
	double t[MAX_NUM_ZENITH_DIVISION];

    double s_angle = CI_PI*ci_config_geometry->starting_zenith_angle/180.;			               /* convert to rad */
    double d_angle = CI_PI*(ci_config_geometry->ending_zenith_angle-ci_config_geometry->starting_zenith_angle)/ci_config_geometry->num_of_zenith_division/180.;
    int i;
	for( i=0; i<ci_config_geometry->num_of_zenith_division; i++ ) {
		//elevation angle is 90 - zenith angle. Changed to Zenith angle 6/16/2000
		zenith_ang[i] = /*CI_PI/2. - */(s_angle + d_angle * ( i + 0.5 ));
		z[i] = ci_tan(zenith_ang[i]);
		if(ci_result.tranmission_efficient[i]!=0) /* changed for WinME exceptions */
			t[i] = ci_log(ci_result.tranmission_efficient[i]); /* natural log */
		else
			t[i] = -8.6;
		//printf("zenith angle: %f, transmission_efficient %f.", zenith_ang[i], ci_result.tranmission_efficient[i]);
    }

/*

	CIResultDialog dlg1;
	char result[10240];
	char line[256];
	memset(result, 0, 10240);
	memset(line, 0, 256);
	strcat(result, "Results:\r\n");


	for( i=0; i<ci_config_geometry->num_of_zenith_division; i++ ) {
		//elevation angle is 90 - zenith angle. Changed to Zenith angle 6/16/2000
		sprintf(line,"i %d, ZA %f, z[i] %f, te %f, da %f, n %d, t %f\r\n",i,zenith_ang[i],z[i],ci_result.tranmission_efficient[i],d_angle,ci_config_geometry->num_of_zenith_division,t[i]);
		strcat(result, line);
   }
	dlg1.m_ResultText = CString(result);
	dlg1.DoModal();
	dlg1.DestroyWindow();
*/


	/* calculate the transmission coefficient for diffuse penetration */
    ci_result.trans_coeff_diffuse = 0;
    for( i=0; i<ci_config_geometry->num_of_zenith_division; i++ )
        ci_result.trans_coeff_diffuse += (ci_result.tranmission_efficient[i]*ci_sin(zenith_ang[i])*ci_cos(zenith_ang[i]));
    ci_result.trans_coeff_diffuse *= (2.*d_angle);

	/* find x using bisection method */
    double xmax = 10.0, xmin = 0.1, x = 1.0;
	double s1, s2, s3, s4;
	double tz, kb, ktmp, dk, f;
    double      dx = 0.01;
    do {
		s1 = 0; s2 = 0; s3 = 0; s4 = 0;
		for( int j=0; j<ci_config_geometry->num_of_zenith_division; j++ ) {
         tz = z[j];
			ktmp = fnk( tz, x );
			kb = ktmp;
			ktmp = fnk( tz, x+dx );
			dk = ktmp - kb;

			s1 += kb * t[j];
			s2 += kb * kb;
			s3 += kb * dk;
			s4 += dk * t[j];
		}
		f = s2 * s4 - s1 * s3; /* this will always compute to close to zero???? As x increases f goes to zero*/
		if( f < 0 ) xmin = x;
		else        xmax = x;
		x = ( xmax + xmin ) / 2.0;
    } while( xmax - xmin > 0.01 );

	ci_result.lai = -s1/s2;
	ci_result.mean_leaf_angle = 90.0*(0.1+0.9*ci_exp((double)(-0.5*x)));


/*


	CIResultDialog dlg1;
	char result[10240];
	char line[256];
	memset(result, 0, 10240);
	memset(line, 0, 256);
       		sprintf(line," s1 %g, s2 %g, s3 %g, s4 %g, x %g, f %g\r\n",s1,s2,s3,s4,x,f);
		strcat(result, line);
		for( int j=0; j<ci_config_geometry->num_of_zenith_division; j++ ) {
      sprintf(line,"ZenithAngle %g, MeasuredT %g, PredictedT %g\r\n",180/3.14159*ci_atan(z[j]),ci_exp(t[j]),ci_exp(-fnk(z[j],x)*ci_result.lai));
		strcat(result, line);
      }
	dlg1.m_ResultText = CString(result);
	dlg1.DoModal();
	dlg1.DestroyWindow();



*/









	ci_result.x = x;

	::SetCursor(::LoadCursor(NULL, IDC_ARROW));

	ShowResult();
}

BOOL CDibDoc::IsMeasurable()
{
	return (state == CI_SHOWCROPPEDIMAGE || state == CI_SHOWLEAFIMAGE);
}

void CDibDoc::ShowResult()
{
	CIResultDialog dlg;
	char result[10240];
	char line[256];
	memset(result, 0, 10240);
	memset(line, 0, 256);
	strcat(result, "Results:\r\n");
	BOOL bHasResult = FALSE;
	if (ci_config_output->bLAI)
	{
		sprintf(line, "Leaf Area Index:\t\t%f\r\n", ci_result.lai);
		strcat(result, line);
		bHasResult = TRUE;
	}
	if (ci_config_output->bMFIA)
	{
		sprintf(line, "Mean Leaf Angle:\t\t%f\r\n", ci_result.mean_leaf_angle);
		strcat(result, line);
		bHasResult = TRUE;
	}
	if (ci_config_output->bTD)
	{
		sprintf(line, "Trans. Coef. for defuse penetration:\t%f\r\n\r\n", ci_result.trans_coeff_diffuse);
		strcat(result, line);
		bHasResult = TRUE;
	}
	double min_angle = CI_PI * ci_config_geometry->starting_zenith_angle / 180.;
	double max_angle = CI_PI * ci_config_geometry->ending_zenith_angle / 180.;
	double d_angle = (max_angle-min_angle)/ci_config_geometry->num_of_zenith_division;
	if (ci_config_output->bTR)
	{
		strcat(result, "Transm. coef. for radiation penetration (T) : \r\n");
//		strcat(result, "Zenith_div\tZenith_angle\tMeasured_T\tPredicted_T\r\n");
		strcat(result, "Zenith_div\tZenith_angle\tMeasured_T\r\n");
		for( int i=0; i<ci_config_geometry->num_of_zenith_division; i++ )
		{
			double everageAngle = min_angle + (i+0.5) * d_angle;
			double expected = ci_exp( - fnk(ci_tan(everageAngle),ci_result.x) * ci_result.lai );
//			sprintf( line,"[%2d]\t\t%6.2lf\t\t%6.2lf\t\t%6.2lf\r\n",
//				i, (180 * everageAngle / CI_PI),
//				ci_result.tranmission_efficient[i], expected );
			sprintf( line,"[%2d]\t\t%6.2lf\t\t%6.2lf\r\n",
				i, (180 * everageAngle / CI_PI),
				ci_result.tranmission_efficient[i], expected );
			strcat(result, line);
		}
		bHasResult = TRUE;
	}
	if (ci_config_output->bK)
	{
		sprintf( line,	    "\r\nExtinction coefficient (K) : \r\n");
		strcat(result, line);
		sprintf( line,	    "Zenith_div\tZenith_angle\tMeasured_K \r\n");
		strcat(result, line);
		for(int i=0; i<ci_config_geometry->num_of_zenith_division; i++ )
		{
			double everageAngle = min_angle + (i+0.5) * d_angle;
			double expected = fnk(ci_tan(everageAngle),ci_result.x);
			sprintf( line, "[%2d]\t\t%6.2lf\t\t%6.2lf\r\n",
			i, (180 * everageAngle / CI_PI), expected );
			strcat(result, line);
		}
		bHasResult = TRUE;
	}
	if (ci_config_output->bLD)
	{
		sprintf( line,	    "\r\nLeaf Distribution (LD): \r\n");
		strcat(result, line);
		sprintf( line,	    "Azimuthal_div\tStart_angle\tEnd_angle\tMeasured_D\r\n");
		strcat(result, line);
		for(int i=0; i<ci_config_geometry->num_of_azmuthal_division; i++ )
		{
			if (IsInBlankoutRegion(i))
			{
				sprintf( line, "[%2d]\t\t%6.2f\t\t%6.2f\t\tBlanked out\r\n",
						i, 
						(float)i * 360 / ci_config_geometry->num_of_azmuthal_division, 
						(float)(i+1) * 360 / ci_config_geometry->num_of_azmuthal_division);
				strcat(result, line);
			}
			else
			{
				sprintf( line, "[%2d]\t\t%6.2f\t\t%6.2f\t\t%5.2f\r\n",
						i, 
						(float)i * 360 / ci_config_geometry->num_of_azmuthal_division, 
						(float)(i+1) * 360 / ci_config_geometry->num_of_azmuthal_division,
						ci_result.leaf_distribution[i] );
				strcat(result, line);
			}
		}
		bHasResult = TRUE;
	}
	if (ci_config_output->bPAR)
	{
		sprintf( line,		"\r\nPAR: ");
		strcat(result, line);
		sprintf( line,		"   %6.1f         Sunflecks %6.1f%%\r\n",::PAR,::Flecks);
		strcat(result, line);
		bHasResult = TRUE;
	}

	if (!bHasResult)
		strcat(result, "You have chosen not to display any result.");
	dlg.m_ResultText = CString(result);
	dlg.DoModal();
	dlg.DestroyWindow();
}

void CDibDoc::ShowResultSF()
{
	CIResultDialog dlg;
	char result[10240];
	char line[256];
	memset(result, 0, 10240);
	memset(line, 0, 256);
	strcat(result, "Results:\r\n");
	BOOL bHasResult = FALSE;
	double min_angle = CI_PI * ci_config_geometry->starting_zenith_angle / 180.;
	double max_angle = CI_PI * ci_config_geometry->ending_zenith_angle / 180.;
	double d_angle = (max_angle-min_angle)/ci_config_geometry->num_of_zenith_division;
	if (ci_config_output->bPAR)
	{
		sprintf( line,		"\r\nPAR: ");
		strcat(result, line);
		sprintf( line,		"   %6.1f         Sunflecks %6.1f%%\r\n",::PAR,::Flecks);
		strcat(result, line);
		bHasResult = TRUE;
	}

	if (!bHasResult)
		strcat(result, "You have chosen not to display any result.");
	dlg.m_ResultText = CString(result);
	dlg.DoModal();
	dlg.DestroyWindow();
}

void CDibDoc::PrintResult(CDC* pDC)
{
	/*
	char line[80];
	int y = 0;
	pDC->TextOut(0, y, "Results:");
	sprintf(line, "Leaf Area Index: %f", ci_result.lai);
	y += 20;
	pDC->TextOut(0, y, line);
	sprintf(line, "Mean Leaf Angle: %f", ci_result.mean_leaf_angle);
	y += 20;
	pDC->TextOut(0, y, line);
	sprintf(line, "Trans. Coef. for defuse penetration: %f", ci_result.trans_coeff_diffuse);
	y += 20;
	pDC->TextOut(0, y, line);
	y += 20;
	pDC->TextOut(0, y, "Transm. coef. for radiation penetration (T) : ");
	y += 20;
	pDC->TextOut(0, y, "Zenith_div  Zenith_angle  Measured_T  Predicted_T");
	for( int i=0; i<ci_config_geometry->num_of_zenith_division; i++ )
	{
		double everageAngle = (i+0.5) * CI_PI * 90 / 180 / ci_config_geometry->num_of_zenith_division;
		double expected = ci_exp( - fnk(ci_tan(everageAngle),ci_result.x) * ci_result.lai );
	    sprintf( line,"[%2d]       %6.2lf       %6.2lf      %6.2lf",
			i, (i+0.5) * 90 / ci_config_geometry->num_of_zenith_division,
			ci_result.tranmission_efficient[i], expected );
		y += 20;
		pDC->TextOut(0, y, line);
	}

	sprintf( line,	    "Extinction coefficient (K) : ");
	y += 20;
	pDC->TextOut(0, y, line);
	sprintf( line,	    "Zenith_div  Zenith_angle  Measured_K ");
	y += 20;
	pDC->TextOut(0, y, line);
	for( i=0; i<ci_config_geometry->num_of_zenith_division; i++ )
	{
		double everageAngle = (i+0.5) * CI_PI * 90 / 180 / ci_config_geometry->num_of_zenith_division;
		double expected = fnk(ci_tan(everageAngle),ci_result.x);
	    sprintf( line, "[%2d]       %6.2lf       %6.2lf",
	    i, (i+0.5) * 90 / ci_config_geometry->num_of_zenith_division, expected );
		y += 20;
		pDC->TextOut(0, y, line);
	}
	sprintf( line,	    "Leaf Distribution (D): ");
	y += 20;
	pDC->TextOut(0, y, line);
	sprintf( line,	    "Azimuthal_div  Start_angle  End_angle  Measured_D");
	y += 20;
	pDC->TextOut(0, y, line);
	for( i=0; i<ci_config_geometry->num_of_azmuthal_division; i++ )
	{
		sprintf( line, "[%2d]         %6.2f          %6.2f     %5.2f",
					i, 
					(float)i * 360 / ci_config_geometry->num_of_azmuthal_division, 
					(float)(i+1) * 360 / ci_config_geometry->num_of_azmuthal_division,
					ci_result.leaf_distribution[i] );
		y += 20;
		pDC->TextOut(0, y, line);
	}
	*/
}

double CDibDoc::fnk( double z, double x )
{
    return( ci_sqrt(x*x+(z*z)) / (x+1.774*ci_pow(x+1.182, -0.733)) );
}

double CDibDoc::fnk2( double z, double x )
{
    return( ci_sqrt(x*x+1/(tan(1.570795-atan(z))*tan(1.570795-atan(z)))) / (1.47+(((.000509*x-.013)*x+.1223)*x+.45)*x));
}

void CDibDoc::SetDrawGeoDiv(BOOL bDraw)
{
	if (bDraw != m_drawGeometryDivision)
	{
		m_drawGeometryDivision = bDraw;
	}
}

