// DibDoc.h : interface of the CDibDoc class
//
/////////////////////////////////////////////////////////////////////////////
#ifndef _CI_DIBDOC_H_
#define _CI_DIBDOC_H_

#include "CiResult.h"

typedef struct{
	BYTE b;
	BYTE g;
	BYTE r;
} COLORPOINT;

typedef struct{
	int dist_r[256];
	int dist_g[256];
	int dist_b[256];
} HISTOGRAM;

typedef struct{
	int index_left_half;
	int index_max;
	int index_right_half;
	int	max_value;
	int	count;
} PULSE;

//the angle, radius and value of four corners in a rectangle.
typedef struct{
	int f00;
	int f01;
	int f10;
	int f11;
	int r0;
	int r1;
	int a0;
	int a1;
} THRESH_RECT;

//state constants
typedef enum {
	CI_INITIALIZED,
	CI_SHOWRAWIMAGE,
	CI_SHOWCROPPEDIMAGE,
	CI_SHOWLEAFIMAGE,
} CiState;

//image format constance
typedef enum {
	CI_UNKNOWN_FORMAT,
	CI_BITMAP_FORMAT,
	CI_JPEG_FORMAT,
}CiImageFormat;

#define MAX_IMAGE_HEIGHT	640

class CDibDoc : public CDocument
{
protected: // create from serialization only
	CDibDoc();
	DECLARE_DYNCREATE(CDibDoc)

// Attributes

// Operations
public:

	void SetOriginalDib(HGLOBAL hdib);
	HGLOBAL GetOriginalDib(void) { return original_hdib; }

	unsigned RealizePalette(CDC* pDC);

	void OnDraw(CDC* pDC, CRect rc);

	void	SetDrawGeoDiv(BOOL b);

	//whether a point is in the cropping circle.
	BOOL	IsInCropCircle(int x, int y);
	int		Radius(int i, int j);
	//which ring is the point in.
	int		RingIndex(int x, int y);
	//angle in [0, 360)
	int		Angle(int x, int y);
	//whether a point is a leaf pixel
	int		AngleIndex(int x, int y);
	//whether a point is a leaf pixel
	BOOL	IsLeaf(int x, int y);
	//if the pixel is in blankout region or not
	//param x and y are in device cordinates.
	BOOL	IsInBlankoutRegion(int x, int y);
	//param i is the azmuthal region index
	BOOL	IsInBlankoutRegion(int i);

	//blanck out the pixels outside the cropping circle and
	//initialize the starting and ending indices for the circle.
	void	OnCrop(HGLOBAL cropping_hdib);
	//whether the current image can be cropped or not.
	BOOL	IsCropable();
	//update the images using the input image as original.
	void	UpdateImages();
	void	BlankoutImage();
	void	UpdateScanIndex();

	//measure the cropped image
	void	Measure();
	//whether the current image can be measured.
	BOOL	IsMeasurable();

	double	fnk(double dz, double x);
	double	fnk2(double dz, double x);
	//calculate the transmissiont coeff for all the rings.
	void	Get_trans_coeff();
	//calcualte the leaf percentage in all angular area.
	void	Get_leaf_dist();

	void	PrintResult(CDC* pDC);

	void	SetState(CiState s);
	CiState	GetState() { return state; }

	void	DrawGeometryDivision(CDC* pDC, CRect rc);

	//calculate whether a pixel is leaf or not and set the pixels for leaf.
	void	GetLeafDib();

	//peak method.
	void	GetLeafIntensity(int *hist, int &cutoff);
	void	GetLeafDib_Peak();
	int		GetLeafCutoffIntensity_Peak();

	//multi threshhood method.
	void	HistogramSingle(int *hist, int &totalNum);
	int		Middle( int *hist, int total, int cutoffPcntLow, int cutoffPcntHigh,
					int cutoffPcnt, int &minIntensity, int &maxIntensity );
	void	GetLeafDib_MultiThresh();
	void	GetDivisionRegion_MultiThresh(int i, int j, int& azmuthalIndex, int& zenithIndex);
	int		GetCutoffIntensity_MultiThresh(THRESH_RECT *tr, int i, int j);

	//set the pixel to be leaf.
	void	SetLeaf(int i, int j);
	//set the pixel to be not leaf.
	void	SetNotLeaf(int i, int j);

	//set the pixel to be blankout
	void	SetBlankoutRegion(int i, int j);

	//show the result.
	void	ShowResult();
	void	ShowResultSF();
	void	ShowHistogram(char* title, int *hist);

	void	GetMaximums(int *hist, int sizeH, PULSE *pulses, int sizeP, int& numOfPulses);
	int		GetIndexOfMinimum(int *smoothed_intensity, int i0, int i1);

	//create a copy for the input bitmap
	HGLOBAL copyBitMap(HGLOBAL	src_hdib);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDibDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CDibDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	//the state of the UI. one of the predefined states.
	CiState		state;

	//flag to draw the geometry division.
	BOOL	m_drawGeometryDivision;

	//the crop circle start/end indices.
	int		CropCircleStartIndex[MAX_IMAGE_HEIGHT];
	int		CropCircleEndIndex[MAX_IMAGE_HEIGHT];

	/*
	* a horizontal scan line can be broken into two segments due to the 
	* blankout region and starting zenith angle.
	*/
	//scan line part 1 start position in bitmap.
	int		StartIndexScan1[MAX_IMAGE_HEIGHT];
	//scan line part 1 end position in bitmap.
	int		EndIndexScan1[MAX_IMAGE_HEIGHT];
	//scan line part 2 start position in bitmap.
	int		StartIndexScan2[MAX_IMAGE_HEIGHT];
	//scan line part 2 end position in bitmap.
	int		EndIndexScan2[MAX_IMAGE_HEIGHT];

	//blankout region starting index in bitmap.
	int		BlankoutStartI[MAX_IMAGE_HEIGHT];
	//blankout region ending index in bitmap.
	int		BlankoutEndI[MAX_IMAGE_HEIGHT];
	//blankout region starting index in bitmap.
	int		BlankoutStartI2[MAX_IMAGE_HEIGHT];
	//blankout region ending index in bitmap.
	int		BlankoutEndI2[MAX_IMAGE_HEIGHT];

	//the starting pointers for rows of the cropped bitmap.
	COLORPOINT* croppedDibRows[MAX_IMAGE_HEIGHT];
	//the starting pointers for rows of the leaf bitmap.
	COLORPOINT* leafDibRows[MAX_IMAGE_HEIGHT];

	//the result of statistic caculation.
	CiResult	ci_result;

	//used for multi-histogram algorithm.
	int		cutoffIntensitys[MAX_NUM_ZENITH_DIVISION][MAX_NUM_AZMUTHAL_DIVISION];

// Generated message map functions
public:
	
	//the original bitmap.
	HGLOBAL		original_hdib;
	//current bitmap, after all kinds of operation.
	HGLOBAL		cropped_hdib;
	//bit maps for black and white leaf.
	HGLOBAL		leaf_hdib;

	BOOL		saveProcessedImageWhenSave;

	BOOL		m_drawBlankoutAsLines;

	//the ratio between the scales of view region and image resion.
	//assume x y have the same scale.
	float		viewImageScale;

	BOOL		m_fillWhenPrint;

	//{{AFX_MSG(CDibDoc)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#endif
/////////////////////////////////////////////////////////////////////////////
